Package: netropy
Title: Statistical Entropy Analysis of Network Data
Version: 0.3.0
Authors@R: 
    person(given = "Termeh",
           family = "Shafie",
           role = c("aut", "cre"),
           email = "termeh.shafie@uni-konstanz.de")
Description: Statistical entropy analysis of network data as introduced by Frank and Shafie (2016) <doi:10.1177/0759106315615511>, and a in textbook which is in progress.
License: MIT + file LICENSE
Encoding: UTF-8
LazyData: true
Imports: ggraph, ggplot2, igraph
RoxygenNote: 7.3.2
Language: en-US
Depends: R (>= 3.6)
Suggests: testthat (>= 3.0.0), rmarkdown, knitr
URL: https://github.com/termehs/netropy
Config/testthat/edition: 3
VignetteBuilder: knitr
NeedsCompilation: no
Packaged: 2026-04-24 08:32:36 UTC; termehshafie
Author: Termeh Shafie [aut, cre]
Maintainer: Termeh Shafie <termeh.shafie@uni-konstanz.de>
Repository: CRAN
Date/Publication: 2026-04-24 08:50:02 UTC
Built: R 4.6.0; ; 2026-04-25 13:47:39 UTC; unix

# Generated by roxygen2: do not edit by hand

export(assoc_graph)
export(div_gof)
export(entropy_bivar)
export(entropy_tetravar)
export(entropy_trivar)
export(get_dyad_var)
export(get_triad_var)
export(joint_entropy)
export(make_pred_plot)
export(prediction_power)
export(redundancy)
importFrom(ggplot2,.data)

<!-- README.md is generated from README.Rmd. Please edit that file -->

# Package overview: `netropy` <img src="man/figures/hex_netropy.png" align="right" width="300px"/>

[![CRAN
status](https://www.r-pkg.org/badges/version/netropy)](https://cran.r-project.org/package=netropy)
[![CRAN
Downloads](http://cranlogs.r-pkg.org/badges/netropy)](https://CRAN.R-project.org/package=netropy)

## Installation

You can install the released version of netropy from
[CRAN](https://CRAN.R-project.org) with:

``` r
install.packages("netropy")
```

The development version from [GitHub](https://github.com/) with:

``` r
# install.packages("devtools")
devtools::install_github("termehs/netropy")
```

## Statistical Entropy Analysis of Network Data

Multivariate entropy analysis is a general statistical method for
analyzing and finding dependence structure in data consisting of
repeated observations of variables with a common domain and with
discrete finite range spaces. Only nominal scale is required for each
variable, so only the size of the variable’s range space is important
but not its actual values. Variables on ordinal or numerical scales,
even continuous numerical scales, can be used, but they should be
aggregated so that their ranges match the number of available repeated
observations. By investigating the frequencies of occurrences of joint
variable outcomes, complicated dependence structures, partial
independence and conditional independence as well as redundancies and
functional dependence can be found.

This package introduces these entropy tools in the context of network
data. Brief description of various functions implemented in the package
are given in the following but more details are provided in the package
vignettes and the references listed.

``` r
library('netropy')
```

## Loading Internal Data

The different entropy tools are explained and illustrated by exploring
data from a network study of a corporate law firm, which has previously
been analysed by several authors
([link](https://www.stats.ox.ac.uk/~snijders/siena/Lazega_lawyers_data.htm)).
The data set is included in the package as a list with objects
representing adjacency matrices for each of the three networks advice
(directed), friendship (directed) and co-work (undirected), together
with a data frame comprising 8 attributes on each of the 71 lawyers.

To load the data, extract each object and assign the correct names to
them:

``` r
data(lawdata) 
adj.advice <- lawdata[[1]]
adj.friend <- lawdata[[2]]
adj.cowork <-lawdata[[3]]
df.att <- lawdata[[4]]
```

## Variable Domains and Data Editing

A requirement for the applicability of these entropy tools is the
specification of discrete variables with finite range spaces on the same
domain: either node attributes/vertex variables, edges/dyad variables or
triad variables. These can be either observed or transformed as shown in
the following using the above example data set.

We have 8 vertex variables with 71 observations, two of which (`years`
and `age`) are numerical and needs categorization based on their
cumulative distributions. This categorization is in details described in
the vignette “variable domains and data editing”. Here we just show the
new dataframe created (note that variable `senior` is omitted as it only
comprises unique values and that we edit all variable to start from 0):

``` r
att.var <-
  data.frame(
    status   = df.att$status-1,
    gender   = df.att$gender,
    office   = df.att$office-1,
    years    = ifelse(df.att$years <= 3,0,
                      ifelse(df.att$years <= 13,1,2)),
    age      = ifelse(df.att$age <= 35,0,
                      ifelse(df.att$age <= 45,1,2)),
    practice = df.att$practice,
    lawschool= df.att$lawschool-1
    )
head(att.var)
#>   status gender office years age practice lawschool
#> 1      0      1      0     2   2        1         0
#> 2      0      1      0     2   2        0         0
#> 3      0      1      1     1   2        1         0
#> 4      0      1      0     2   2        0         2
#> 5      0      1      1     2   2        1         1
#> 6      0      1      1     2   2        1         0
```

These vertex variables can be transformed into dyad variables by using
the function `get_dyad_var()`. Observed node attributes in the dataframe
`att_var` are then transformed into pairs of individual attributes. For
example, `status` with binary outcomes is transformed into dyads having
4 possible outcomes $(0,0), (0,1), (1,0), (1,1)$:

``` r
dyad.status    <- get_dyad_var(att.var$status, type = 'att')
dyad.gender    <- get_dyad_var(att.var$gender, type = 'att')
dyad.office    <- get_dyad_var(att.var$office, type = 'att')
dyad.years     <- get_dyad_var(att.var$years, type = 'att')
dyad.age       <- get_dyad_var(att.var$age, type = 'att')
dyad.practice  <- get_dyad_var(att.var$practice, type = 'att')
dyad.lawschool <- get_dyad_var(att.var$lawschool, type = 'att')
```

Similarly, dyad variables can be created based on observed ties. For the
undirected edges, we use indicator variables read directly from the
adjacency matrix for the dyad in question, while for the directed ones
(`advice` and `friendship`) we have pairs of indicators representing
sending and receiving ties with 4 possible outcomes :

``` r
dyad.cwk    <- get_dyad_var(adj.cowork, type = 'tie')
dyad.adv    <- get_dyad_var(adj.advice, type = 'tie')
dyad.frn    <- get_dyad_var(adj.friend, type = 'tie')
```

All 10 dyad variables are merged into one data frame for subsequent
entropy analysis:

``` r
dyad.var <-
  data.frame(cbind(status   = dyad.status$var,
                  gender    = dyad.gender$var,
                  office    = dyad.office$var,
                  years     = dyad.years$var,
                  age       = dyad.age$var,
                  practice  = dyad.practice$var,
                  lawschool = dyad.lawschool$var,
                  cowork    = dyad.cwk$var,
                  advice    = dyad.adv$var,
                  friend    = dyad.frn$var)
                  )
head(dyad.var)
#>   status gender office years age practice lawschool cowork advice friend
#> 1      3      3      0     8   8        1         0      0      3      2
#> 2      3      3      3     5   8        3         0      0      0      0
#> 3      3      3      3     5   8        2         0      0      1      0
#> 4      3      3      0     8   8        1         6      0      1      2
#> 5      3      3      0     8   8        0         6      0      1      1
#> 6      3      3      1     7   8        1         6      0      1      1
```

A similar function `get_triad_var()` is implemented for transforming
vertex variables and different relation types into triad variables. This
is described in more detail in the vignette “variable domains and data
editing”.

## Univariate, Bivariate and Trivariate Entropies

The function `entropy_bivar()` computes the bivariate entropies of all
pairs of variables in the dataframe. The output is given as an upper
triangular matrix with cells giving the bivariate entropies of row and
column variables. The diagonal thus gives the univariate entropies for
each variable in the dataframe:

``` r
H2 <- entropy_bivar(dyad.var)
H2
#>           status gender office years   age practice lawschool cowork advice
#> status     1.493  2.868  3.640 3.370 3.912    3.453     4.363  2.092  2.687
#> gender        NA  1.547  3.758 3.939 4.274    3.506     4.439  2.158  2.785
#> office        NA     NA  2.239 4.828 4.901    4.154     5.058  2.792  3.388
#> years         NA     NA     NA 2.671 4.857    4.582     5.422  3.268  3.868
#> age           NA     NA     NA    NA 2.801    4.743     5.347  3.411  4.028
#> practice      NA     NA     NA    NA    NA    1.962     4.880  2.530  3.127
#> lawschool     NA     NA     NA    NA    NA       NA     2.953  3.567  4.186
#> cowork        NA     NA     NA    NA    NA       NA        NA  0.615  1.687
#> advice        NA     NA     NA    NA    NA       NA        NA     NA  1.248
#> friend        NA     NA     NA    NA    NA       NA        NA     NA     NA
#>           friend
#> status     2.324
#> gender     2.415
#> office     3.044
#> years      3.483
#> age        3.637
#> practice   2.831
#> lawschool  3.812
#> cowork     1.456
#> advice     1.953
#> friend     0.881
```

Bivariate entropies can be used to detect redundant variables that
should be omitted from the dataframe for further analysis. This occurs
when the univariate entropy for a variable is equal to the bivariate
entropies for pairs including that variable. As seen above, the
dataframe `dyad.var` has no redundant variables. This can also be
checked using the function `redundancy()` which yields a binary matrix
as output indicating which row and column variables are hold the same
information:

``` r
redundancy(dyad.var)
#> no redundant variables
#> NULL
```

More examples of using the function `redundancy()` is given in the
vignette “univariate bivariate and trivariate entropies”.

Trivariate entropies can be computed using the function
`entropy_trivar()` which returns a dataframe with the first three
columns representing possible triples of variables `V1`,`V2`, and `V3`
from the dataframe in question, and their entropies `H(V1,V2,V3)` as the
fourth column. We illustrated this on the dataframe `dyad.var`:

``` r
H3 <- entropy_trivar(dyad.var)
head(H3, 10) # view first 10 rows of dataframe
#>        V1     V2        V3 H(V1,V2,V3)
#> 1  status gender    office       4.938
#> 2  status gender     years       4.609
#> 3  status gender       age       5.129
#> 4  status gender  practice       4.810
#> 5  status gender lawschool       5.664
#> 6  status gender    cowork       3.464
#> 7  status gender    advice       4.048
#> 8  status gender    friend       3.685
#> 9  status office     years       5.321
#> 10 status office       age       5.721
```

## Joint Entropy and Association Graphs

Joint entropies is a non-negative measure of association among pairs of
variables. It is equal to 0 if and only if two variables are completely
independent of each other.

The function `joint_entropy()` computes the joint entropies between all
pairs of variables in a given dataframe and returns a list consisting of
the upper triangular joint entropy matrix (univariate entropies in the
diagonal) and a dataframe giving the frequency distributions of unique
joint entropy values. A function argument specifies the precision given
in number of decimals for which the frequency distribution of unique
entropy values is created (default is 3). Applying the function on the
dataframe `dyad.var` with two decimals:

``` r
J <- joint_entropy(dyad.var, 2)
J$matrix
#>           status gender office years  age practice lawschool cowork advice
#> status      1.49   0.17   0.09  0.79 0.38     0.00      0.08   0.02   0.05
#> gender        NA   1.55   0.03  0.28 0.07     0.00      0.06   0.00   0.01
#> office        NA     NA   2.24  0.08 0.14     0.05      0.13   0.06   0.10
#> years         NA     NA     NA  2.67 0.61     0.05      0.20   0.02   0.05
#> age           NA     NA     NA    NA 2.80     0.02      0.41   0.01   0.02
#> practice      NA     NA     NA    NA   NA     1.96      0.04   0.05   0.08
#> lawschool     NA     NA     NA    NA   NA       NA      2.95   0.00   0.01
#> cowork        NA     NA     NA    NA   NA       NA        NA   0.62   0.18
#> advice        NA     NA     NA    NA   NA       NA        NA     NA   1.25
#> friend        NA     NA     NA    NA   NA       NA        NA     NA     NA
#>           friend
#> status      0.05
#> gender      0.01
#> office      0.08
#> years       0.07
#> age         0.05
#> practice    0.01
#> lawschool   0.02
#> cowork      0.04
#> advice      0.18
#> friend      0.88
J$freq
#>       j  #(J = j) #(J >= j)
#> 1  0.79         1         1
#> 2  0.61         1         2
#> 3  0.41         1         3
#> 4  0.38         1         4
#> 5  0.28         1         5
#> 6   0.2         1         6
#> 7  0.18         2         8
#> 8  0.17         1         9
#> 9  0.14         1        10
#> 10 0.13         1        11
#> 11  0.1         1        12
#> 12 0.09         1        13
#> 13 0.08         4        17
#> 14 0.07         2        19
#> 15 0.06         2        21
#> 16 0.05         7        28
#> 17 0.04         2        30
#> 18 0.03         1        31
#> 19 0.02         5        36
#> 20 0.01         5        41
#> 21    0         4        45
```

As seen, the strongest association is between the variables `status` and
`years` with joint entropy values of 0.79. We have independence (joint
entropy value of 0) between two pairs of variables:
(`status`,`practice`), (`practise`,`gender`), (`cowork`,`gender`),and
(`cowork`,`lawschool`).

These results can be illustrated in a association graph using the
function `assoc_graph()` which returns a `ggraph` object in which nodes
represent variables and links represent strength of association (thicker
links indicate stronger dependence). To use the function we need to load
the `ggraph` library and to determine a threshold which the graph drawn
is based on. We set it to 0.15 so that we only visualize the strongest
associations

``` r
library(ggraph)
assoc_graph(dyad.var, 0.15)
```

<img src="man/figures/README-assoc_g-1.png" width="100%" style="display: block; margin: auto;" />

Given this threshold, we see isolated and disconnected nodes
representing independent variables. We note strong dependence between
the three dyadic variables `status`,`years` and `age`, but also a
somewhat strong dependence among the three variables `lawschool`,
`years` and `age`, and the three variables `status`, `years` and
`gender`. The association graph can also be interpreted as a tendency
for relations `cowork` and `friend` to be independent conditionally on
relation `advice`, that is, any dependence between dyad variables
`cowork` and `friend` is explained by `advice`.

A threshold that gives a graph with reasonably many small independent or
conditionally independent subsets of variables can be considered to
represent a multivariate model for further testing.

More details and examples of joint entropies and association graphs are
given in the vignette “joint entropies and association graphs”.

## Prediction Power Based on Expected Conditional Entropies

The function `prediction_power()` computes prediction power when pairs
of variables in a given dataframe are used to predict a third variable
from the same dataframe. The variable to be predicted and the dataframe
in which this variable also is part of is given as input arguments, and
the output is an upper triangular matrix giving the expected conditional
entropies of pairs of row and column variables (denoted $X$ and $Y$) of
the matrix, i.e. *EH(Z\|X,Y)* where $Z$ is the variable to be predicted.
The diagonal gives *EH(Z\|X)* , that is when only one variable as a
predictor. Note that `NA`’s are in the row and column representing the
variable being predicted.

Assume we are interested in predicting variable `status` (that is
whether a lawyer in the data set is an associate or partner). This is
done by running the following syntax

``` r
prediction_power('status', dyad.var)
#>           status gender office years   age practice lawschool cowork advice
#> status        NA     NA     NA    NA    NA       NA        NA     NA     NA
#> gender        NA  1.375  1.180 0.670 0.855    1.304     1.225  1.306  1.263
#> office        NA     NA  2.147 0.493 0.820    1.374     1.245  1.373  1.325
#> years         NA     NA     NA 2.265 0.573    0.682     0.554  0.691  0.667
#> age           NA     NA     NA    NA 1.877    1.089     0.958  1.087  1.052
#> practice      NA     NA     NA    NA    NA    2.446     1.388  1.459  1.410
#> lawschool     NA     NA     NA    NA    NA       NA     3.335  1.390  1.337
#> cowork        NA     NA     NA    NA    NA       NA        NA  2.419  1.400
#> advice        NA     NA     NA    NA    NA       NA        NA     NA  2.781
#> friend        NA     NA     NA    NA    NA       NA        NA     NA     NA
#>           friend
#> status        NA
#> gender     1.270
#> office     1.334
#> years      0.684
#> age        1.058
#> practice   1.427
#> lawschool  1.350
#> cowork     1.411
#> advice     1.407
#> friend     3.408
```

For better readability, the powers of different predictors can be
conveniently compared by using prediction plots that display a color
matrix with rows for $X$ and columns for $Y$ with darker colors in the
cells when we have higher prediction power for $Z$. This is shown for
the prediction of `status`:

<img src="man/figures/predplot-status.png" align="center"/>

Obviously, the darkest color is obtained when the variable to be
predicted is included among the predictors, and the cells exhibit
prediction power for a single predictor on the diagonal and for two
predictors symmetrically outside the diagonal. Some findings are as
follows: good predictors for `status` are given by `years` in
combination with any other variable, and `age` in combination with any
other variable. The best sole predictor is `gender`.

More details and examples of expected conditional entropies and
prediction power are given in the vignette “prediction power based on
expected conditional entropies”.

## Divergence Tests of Goodness of Fit

Occurring cliques in association graphs represent connected components
of dependent variables, and by comparing the graphs for different
thresholds, specific structural models of multivariate dependence can be
suggested and tested. The function `div_gof()` allows such hypothesis
tests for pairwise independence of $X$ and $Y$: $X \bot Y$, and pairwise
independence conditional a third variable $Z$: $X\bot Y|Z$.

To test `friend`$\bot$ `cowork`$|$`advice`, that is whether dyad
variable `friend` is independent of `cowork` given `advice` we use the
function as shown below:

``` r
div_gof(dat = dyad.var, var1 = "friend", var2 = "cowork", var_cond = "advice")
#> the specified model of conditional independence cannot be rejected
#>      D df(D)
#> 1 0.94    12
```

Not specifying argument `var_cond` would instead test
`friend`$\bot$`cowork` without any conditioning.

## References

Parts of the theoretical background is provided in the package
vignettes, but for more details, consult the following literature:

> Frank, O., & Shafie, T. (2016). Multivariate entropy analysis of
> network data. *Bulletin of Sociological Methodology/Bulletin de
> Méthodologie Sociologique*, 129(1), 45-63.
> [link](https://doi.org/10.1177%2F0759106315615511)

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(out.width = "100%", cache = FALSE)

## ----load_library, eval=TRUE--------------------------------------------------
library(netropy)

## ----load_data, eval=TRUE, include=FALSE--------------------------------------
data(lawdata)
adj_advice <- lawdata[[1]]
adj_friend <- lawdata[[2]]
adj_cowork <- lawdata[[3]]
df_att <- lawdata[[4]]

## ----edit_data, eval=TRUE, include=FALSE, results ='hide'---------------------
att_var <- data.frame(
  status    = df_att$status - 1,
  gender    = df_att$gender,
  office    = df_att$office - 1,
  years     = ifelse(df_att$years <= 3, 0,
                ifelse(df_att$years <= 13, 1, 2)),
  age       = ifelse(df_att$age <= 35, 0,
                ifelse(df_att$age <= 45, 1, 2)),
  practice  = df_att$practice,
  lawschool = df_att$lawschool - 1
)

dyad_status    <- get_dyad_var(att_var$status, type = "att")
dyad_gender    <- get_dyad_var(att_var$gender, type = "att")
dyad_office    <- get_dyad_var(att_var$office, type = "att")
dyad_years     <- get_dyad_var(att_var$years, type = "att")
dyad_age       <- get_dyad_var(att_var$age, type = "att")
dyad_practice  <- get_dyad_var(att_var$practice, type = "att")
dyad_lawschool <- get_dyad_var(att_var$lawschool, type = "att")

dyad_cwk <- get_dyad_var(adj_cowork, type = "tie")
dyad_adv <- get_dyad_var(adj_advice, type = "tie")
dyad_frn <- get_dyad_var(adj_friend, type = "tie")

dyad_var <- data.frame(
  status    = dyad_status$var,
  gender    = dyad_gender$var,
  office    = dyad_office$var,
  years     = dyad_years$var,
  age       = dyad_age$var,
  practice  = dyad_practice$var,
  lawschool = dyad_lawschool$var,
  cowork    = dyad_cwk$var,
  advice    = dyad_adv$var,
  friend    = dyad_frn$var
)

## ----show_data, eval=TRUE, include=TRUE---------------------------------------
head(dyad_var)

## ----cond_ind-----------------------------------------------------------------
div_gof(
  dat = dyad_var,
  var1 = "friend",
  var2 = "cowork",
  var_cond = "advice"
)

## ----pair_ind-----------------------------------------------------------------
div_gof(
  dat = dyad_var,
  var1 = "friend",
  var2 = "cowork"
)

## ----unif---------------------------------------------------------------------
div_gof(
  dat = dyad_var,
  var_uniform = "friend"
)

## ----nest_ind1----------------------------------------------------------------
m_full <- list(D = 0, df = 0)

m_reduced <- div_gof(
  dat = dyad_var,
  var1 = "friend",
  var2 = "cowork"
)

div_gof(
  dat = dyad_var,
  model_full = m_full,
  model_reduced = list(D = m_reduced$D, df = m_reduced$df)
)

## ----nest_ind2----------------------------------------------------------------
m_reduced <- div_gof(
  dat = dyad_var,
  var1 = "friend",
  var2 = "cowork",
  var_cond = "advice"
)

div_gof(
  dat = dyad_var,
  model_full = m_full,
  model_reduced = list(D = m_reduced$D, df = m_reduced$df)
)

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(out.width = "100%",
  cache = FALSE
)

## ----load_library, eval=TRUE--------------------------------------------------
library(netropy)

## ----load_data, eval=TRUE, include=FALSE--------------------------------------
data(lawdata) 
adj.advice <- lawdata[[1]]
adj.friend <- lawdata[[2]]
adj.cowork <-lawdata[[3]]
df.att <- lawdata[[4]]

## ----edit_data, eval=TRUE, include=FALSE, results ='hide'---------------------
att.var <-
  data.frame(
    status   = df.att$status-1,
    gender   = df.att$gender,
    office   = df.att$office-1,
    years    = ifelse(df.att$years<=3,0,
                      ifelse(df.att$years<=13,1,2)),
    age      = ifelse(df.att$age<=35,0,
                      ifelse(df.att$age<=45,1,2)),
    practice = df.att$practice,
    lawschool= df.att$lawschool-1
    )
dyad.status    <- get_dyad_var(att.var$status, type = 'att')
dyad.gender    <- get_dyad_var(att.var$gender, type = 'att')
dyad.office    <- get_dyad_var(att.var$office, type = 'att')
dyad.years     <- get_dyad_var(att.var$years, type = 'att')
dyad.age       <- get_dyad_var(att.var$age, type = 'att')
dyad.practice  <- get_dyad_var(att.var$practice, type = 'att')
dyad.lawschool <- get_dyad_var(att.var$lawschool, type = 'att')
dyad.cwk    <- get_dyad_var(adj.cowork, type = 'tie')
dyad.adv    <- get_dyad_var(adj.advice, type = 'tie')
dyad.frn    <- get_dyad_var(adj.friend, type = 'tie')
dyad.var <-
  data.frame(cbind(status    = dyad.status$var,
                  gender    = dyad.gender$var,
                  office    = dyad.office$var,
                  years     = dyad.years$var,
                  age       = dyad.age$var,
                  practice  = dyad.practice$var,
                  lawschool = dyad.lawschool$var,
                  cowork    = dyad.cwk$var,
                  advice    = dyad.adv$var,
                  friend    = dyad.frn$var)
                  )

## ----show_data, eval=TRUE, include=TRUE, results ='markup'--------------------
head(dyad.var)

## ----predpow1, eval=TRUE, include=TRUE----------------------------------------
prediction_power('status', dyad.var)

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(out.width = "100%",
  cache = FALSE
)

## ----eval=TRUE----------------------------------------------------------------
library('netropy')

## ----load_data, eval=TRUE-----------------------------------------------------
data(lawdata) 
adj.advice <- lawdata[[1]]
adj.friend <- lawdata[[2]]
adj.cowork <-lawdata[[3]]
df.att <- lawdata[[4]]

## ----cdf, eval=TRUE, include=TRUE, results ='markup'--------------------------
# for years:
x <- table(df.att$years) 
values<-as.numeric(names(x))
prop.x <- round(prop.table(x),2)
cum.prop=cumsum(prop.x)
frq.years = data.frame(value=values,freq=as.vector(x),
                       rel.freq=as.vector(prop.x), 
                       cum.rel.freq=as.vector(cum.prop))

# for age:
x <- table(df.att$age) 
values<-as.numeric(names(x))
prop.x <- round(prop.table(x),2)
cum.prop=cumsum(prop.x)
frq.age = data.frame(value=values,freq=as.vector(x),
                       rel.freq=as.vector(prop.x), 
                       cum.rel.freq=as.vector(cum.prop))

## ----cdf2, eval=TRUE, include=TRUE, results ='markup'-------------------------
frq.years
frq.age

## ----edit_data1, eval=TRUE, include=TRUE, results ='markup'-------------------
att.var <-
  data.frame(
    status   = df.att$status-1,
    gender   = df.att$gender,
    office   = df.att$office-1,
    years    = ifelse(df.att$years <= 3,0,
                      ifelse(df.att$years <= 13,1,2)),
    age      = ifelse(df.att$age <= 35,0,
                      ifelse(df.att$age <= 45,1,2)),
    practice = df.att$practice,
    lawschool= df.att$lawschool-1
    )
head(att.var)

## ----edit_data2, eval=TRUE, include=TRUE, results ='markup'-------------------
dyad.status    <- get_dyad_var(att.var$status, type = 'att')
dyad.gender    <- get_dyad_var(att.var$gender, type = 'att')
dyad.office    <- get_dyad_var(att.var$office, type = 'att')
dyad.years     <- get_dyad_var(att.var$years, type = 'att')
dyad.age       <- get_dyad_var(att.var$age, type = 'att')
dyad.practice  <- get_dyad_var(att.var$practice, type = 'att')
dyad.lawschool <- get_dyad_var(att.var$lawschool, type = 'att')

## ----edit_data3, eval=TRUE, include=TRUE, message = FALSE, results ='markup'----
dyad.cwk    <- get_dyad_var(adj.cowork, type = 'tie')
dyad.adv    <- get_dyad_var(adj.advice, type = 'tie')
dyad.frn    <- get_dyad_var(adj.friend, type = 'tie')

## ----edit_data4, eval=TRUE, include=TRUE, results ='markup'-------------------
dyad.var <-
  data.frame(cbind(status   = dyad.status$var,
                  gender    = dyad.gender$var,
                  office    = dyad.office$var,
                  years     = dyad.years$var,
                  age       = dyad.age$var,
                  practice  = dyad.practice$var,
                  lawschool = dyad.lawschool$var,
                  cowork    = dyad.cwk$var,
                  advice    = dyad.adv$var,
                  friend    = dyad.frn$var)
                  )
head(dyad.var)

## ----edit_data5, eval=TRUE, include=TRUE, results ='markup'-------------------
triad.status    <- get_triad_var(att.var$status, type = 'att')
triad.gender    <- get_triad_var(att.var$gender, type = 'att')
triad.office    <- get_triad_var(att.var$office, type = 'att')
triad.years     <- get_triad_var(att.var$years, type = 'att')
triad.age       <- get_triad_var(att.var$age, type = 'att')
triad.practice  <- get_triad_var(att.var$practice, type = 'att')
triad.lawschool <- get_triad_var(att.var$lawschool,type = 'att')

## ----edit_data6, eval=TRUE, include=TRUE, message = FALSE, results ='markup'----
triad.cwk    <- get_triad_var(adj.cowork, type = 'tie')
triad.adv    <- get_triad_var(adj.advice, type = 'tie')
triad.frn    <- get_triad_var(adj.friend, type = 'tie')

## ----edit_data7, eval=TRUE, include=TRUE, results ='markup'-------------------
triad.var <- data.frame(cbind(
             status    = triad.status$var,
             gender    = triad.gender$var,
             office    = triad.office$var,
             years     = triad.years$var,
             age       = triad.age$var,
             practice  = triad.practice$var,
             lawschool = triad.lawschool$var,
             cowork    = triad.cwk$var,
             advice    = triad.adv$var,
             friend    = triad.frn$var)
             )
head(triad.var)

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(out.width = "100%",
  cache = FALSE
)

## ----load_library, eval=TRUE--------------------------------------------------
library(netropy)

## ----load_data, eval=TRUE, include=FALSE--------------------------------------
data(lawdata) 
adj.advice <- lawdata[[1]]
adj.friend <- lawdata[[2]]
adj.cowork <-lawdata[[3]]
df.att <- lawdata[[4]]

## ----edit_data, eval=TRUE, include=FALSE, results ='hide'---------------------
att.var <-
  data.frame(
    status   = df.att$status-1,
    gender   = df.att$gender,
    office   = df.att$office-1,
    years    = ifelse(df.att$years<=3,0,
                      ifelse(df.att$years<=13,1,2)),
    age      = ifelse(df.att$age<=35,0,
                      ifelse(df.att$age<=45,1,2)),
    practice = df.att$practice,
    lawschool= df.att$lawschool-1
    )
dyad.status    <- get_dyad_var(att.var$status, type = 'att')
dyad.gender    <- get_dyad_var(att.var$gender, type = 'att')
dyad.office    <- get_dyad_var(att.var$office, type = 'att')
dyad.years     <- get_dyad_var(att.var$years, type = 'att')
dyad.age       <- get_dyad_var(att.var$age, type = 'att')
dyad.practice  <- get_dyad_var(att.var$practice, type = 'att')
dyad.lawschool <- get_dyad_var(att.var$lawschool, type = 'att')
dyad.cwk    <- get_dyad_var(adj.cowork, type = 'tie')
dyad.adv    <- get_dyad_var(adj.advice, type = 'tie')
dyad.frn    <- get_dyad_var(adj.friend, type = 'tie')
dyad.var <-
  data.frame(cbind(status    = dyad.status$var,
                  gender    = dyad.gender$var,
                  office    = dyad.office$var,
                  years     = dyad.years$var,
                  age       = dyad.age$var,
                  practice  = dyad.practice$var,
                  lawschool = dyad.lawschool$var,
                  cowork    = dyad.cwk$var,
                  advice    = dyad.adv$var,
                  friend    = dyad.frn$var)
                  )

## ----show_data, eval=TRUE, include=TRUE, results ='markup'--------------------
head(dyad.var)

## ----joint_ent, eval=TRUE, include=TRUE, results ='markup'--------------------
J <- joint_entropy(dyad.var, 2)
J$matrix
J$freq

## ----assoc_g, eval=TRUE, fig.align='center',fig.width=9, message =FALSE-------
library(ggraph)
assoc_graph(dyad.var, 0.15)

#' @title Bivariate Entropy
#' @description Computes the bivariate entropies between all pairs of (discrete) variables in a multivariate data set.
#' @param dat dataframe with rows as observations and columns as variables. Variables must all be observed or transformed categorical with finite range spaces.
#' @return Upper triangular matrix giving bivariate entropies between pairs of variables given as
#' rows and columns of the matrix. The univariate entropies are given in the diagonal.
#' @details The bivariate entropy \emph{H(X,Y)} of two discrete random variables \emph{X} and \emph{Y}
#' can be used to check for functional relationships and stochastic independence between pairs of variables.
#' The bivariate entropy is bounded according to \cr
#'
#' \emph{H(X) <= H(X,Y) <= H(X) + H(Y)}
#' \cr
#'
#' where \emph{H(X)} and
#' \emph{H(Y)} are the univariate entropies.
#' @author Termeh Shafie
#' @seealso \code{\link{joint_entropy}},  \code{\link{entropy_trivar}}, \code{\link{redundancy}}, \code{\link{prediction_power}}
#' @references Frank, O., & Shafie, T. (2016). Multivariate entropy analysis of network data.
#' \emph{Bulletin of Sociological Methodology/Bulletin de Méthodologie Sociologique}, 129(1), 45-63.
#' @examples
#' # use internal data set
#' data(lawdata)
#' df.att <- lawdata[[4]]
#'
#' # three steps of data editing:
#' # 1. categorize variables 'years' and 'age' based on
#' # approximately three equally size groups (values based on cdf)
#' # 2. make sure all outcomes start from the value 0 (optional)
#' # 3. remove variable 'senior' as it consists of only unique values (thus redundant)
#' df.att.ed <- data.frame(
#'     status = df.att$status,
#'     gender = df.att$gender,
#'     office = df.att$office - 1,
#'     years = ifelse(df.att$years <= 3, 0,
#'         ifelse(df.att$years <= 13, 1, 2)
#'     ),
#'     age = ifelse(df.att$age <= 35, 0,
#'         ifelse(df.att$age <= 45, 1, 2)
#'     ),
#'     practice = df.att$practice,
#'     lawschool = df.att$lawschool - 1
#' )
#'
#' # calculate bivariate entropies
#' H.biv <- entropy_bivar(df.att.ed)
#' # univariate entropies are then given as
#' diag(H.biv)
#' @export
#'
entropy_bivar <- function(dat) {
    varname_orig <- colnames(dat)
    varname_new <- sprintf("V%d", seq_len(length(dat)))
    names(dat) <- varname_new

    H2 <- matrix(0, nrow = ncol(dat), ncol = ncol(dat))

    # rename columns and rows to match variable names in data frame
    colnames(H2) <- colnames(dat)
    rownames(H2) <- colnames(dat)

    # iterate over all variables in data frame to calculate bivariate entropy
    for (x in seq_len(ncol(dat))) {
        for (y in (x):ncol(dat)) {
            freq <- as.data.frame(table(dat[, x], dat[, y]))
            # frequencies of observations ordered in outcome space
            freq.os <- freq[order(freq[, 1], freq[, 2]), ]
            # calculate bivariate entropies
            H2pos <-
                ifelse(freq.os$Freq > 0, freq.os$Freq * log2(freq.os$Freq), 0)
            H2[x, y] <- log2(nrow(dat)) - (1 / nrow(dat)) * (sum(H2pos))
        }
    }

    colnames(H2) <- varname_orig
    rownames(H2) <- varname_orig
    H2[lower.tri(H2)] <- NA
    H2 <- as.matrix(H2)
    H2 <- round(H2, 3)

    return(H2)
}

test_that("bivariate entropy works", {
  data(lawdata)
  df.att <- lawdata[[4]]

  df.att.ed <- data.frame(
    status   = df.att$status,
    gender   = df.att$gender,
    office   = df.att$office-1,
    years    = ifelse(df.att$years<=3,0,
               ifelse(df.att$years<=13,1,2)),
    age      = ifelse(df.att$age<=35,0,
                     ifelse(df.att$age<=45,1,2)),
        practice = df.att$practice,
        lawschool= df.att$lawschool-1)

     H.biv <- entropy_bivar(df.att.ed)
     result <- c(status = 1, gender = 0.817, office = 1.125, years = 1.585,
                 age = 1.585, practice = 0.983, lawschool = 1.533)
     expect_identical(diag(H.biv),result)
})

#' @title Redundant Variables & Dimensionality Reduction
#' @description Finds redundant variables in a dataframe consisting of discrete variables.
#' @param dat dataframe with rows as observations and columns as variables.
#' Variables must all be observed or transformed categorical with finite range spaces.
#' @param dec  the precision given as number of decimals used to round bivariate entropies
#' in order to find redundant variables (the more decimals, the harder to detect redundancy). Default is 3.
#' @return Binary matrix indicating which row and column variables hold the same information.
#' @details   Redundancy is defined as two variables holding the same information (bivariate entropies)
#' as at least one of the variable alone (univariate entropies).
#' Consider removing one of these two variable from the dataframe for further analysis.
#' @author Termeh Shafie
#' @seealso \code{\link{entropy_bivar}},
#' @references Frank, O., & Shafie, T. (2016). Multivariate entropy analysis of network data.
#' \emph{Bulletin of Sociological Methodology/Bulletin de Méthodologie Sociologique}, 129(1), 45-63.
#' @examples
#' # use internal data set
#' data(lawdata)
#' df.att <- lawdata[[4]]
#'
#' # two steps of data editing:
#' # 1. categorize variables 'years' and 'age' based on
#' # approximately three equally size groups (values based on cdf)
#' # 2. make sure all outcomes start from the value 0 (optional)
#' df.att.ed <- data.frame(
#'     senior = df.att$senior,
#'     status = df.att$status,
#'     gender = df.att$gender,
#'     office = df.att$office - 1,
#'     years = ifelse(df.att$years <= 3, 0,
#'         ifelse(df.att$years <= 13, 1, 2)
#'     ),
#'     age = ifelse(df.att$age <= 35, 0,
#'         ifelse(df.att$age <= 45, 1, 2)
#'     ),
#'     practice = df.att$practice,
#'     lawschool = df.att$lawschool - 1
#' )
#'
#' # find redundant variables in dataframe
#' redundancy(df.att.ed) # variable 'senior' should be omitted
#' @export
#'
redundancy <- function(dat, dec = 3) {
    H2 <- entropy_bivar(dat)

    # given input argument dec giving precision, round H2 matrix
    H2 <- round(H2, dec)

    # print out redundant variables
    H2[lower.tri(H2)] <- H2[upper.tri(H2)]
    red <- H2
    for (i in seq_len(ncol(H2))) {
        red[i, ] <- diag(H2)[i] == H2[i, ]
    }
    diag(red) <- 0

    if (sum(red) == 0) {
        message("no redundant variables")
        return()
    }

    return(red)
}

test_that("redundancy works", {
  data(lawdata)
  df.att <- lawdata[[4]]

  df.att.ed <- data.frame(
    senior   = df.att$senior,
    status   = df.att$status,
    gender   = df.att$gender,
    office   = df.att$office-1,
    years    = ifelse(df.att$years<=3,0,
                      ifelse(df.att$years<=13,1,2)),
    age      = ifelse(df.att$age<=35,0,
                      ifelse(df.att$age<=45,1,2)),
    practice = df.att$practice,
    lawschool= df.att$lawschool-1)
  expect_identical(unname(redundancy(df.att.ed)[1,]),c(0,1,1,1,1,1,1,1))
})

#' @title Joint Entropy
#' @description Computes the joint entropies between all pairs of (discrete)
#' variables in a multivariate data set.
#' @param dat dataframe with rows as observations and columns as variables.
#' Variables must all be observed or transformed categorical with finite range spaces.
#' @param dec the precision given in number of decimals for which
#' the frequency distribution of unique entropy values is created. Default is 3.
#' @return List with
#' \item{matrix}{an upper triangular joint entropy matrix (univariate entropies in the diagonal).}
#' \item{freq}{a dataframe giving the frequency distributions of unique joint entropy values.}
#' @details The joint entropy \emph{J(X,Y)} of discrete variables \emph{X} and \emph{Y}
#' is a measure of dependence or association between them, defined as
#' \cr
#'
#' \emph{J(X,Y) = H(X) + H(Y) - H(X,Y)}.
#' \cr
#'
#' Two variables are independent if their joint entropy,
#' i.e. their mutual information, is equal to zero.
#' The frequency distributions can be used to decide upon convenient thresholds for
#' constructing association graphs.
#' @author Termeh Shafie
#' @seealso \code{\link{assoc_graph}}, \code{\link{entropy_bivar}}
#' @references Frank, O., & Shafie, T. (2016). Multivariate entropy analysis of network data.
#' \emph{Bulletin of Sociological Methodology/Bulletin de Méthodologie Sociologique}, 129(1), 45-63.
#' @examples
#' # use internal data set
#' data(lawdata)
#' df.att <- lawdata[[4]]
#'
#' # three steps of data editing:
#' # 1. categorize variables 'years' and 'age' based on
#' # approximately three equally size groups (values based on cdf)
#' # 2. make sure all outcomes start from the value 0 (optional)
#' # 3. remove variable 'senior' as it consists of only unique values (thus redundant)
#' df.att.ed <- data.frame(
#'     status = df.att$status,
#'     gender = df.att$gender,
#'     office = df.att$office - 1,
#'     years = ifelse(df.att$years <= 3, 0,
#'         ifelse(df.att$years <= 13, 1, 2)
#'     ),
#'     age = ifelse(df.att$age <= 35, 0,
#'         ifelse(df.att$age <= 45, 1, 2)
#'     ),
#'     practice = df.att$practice,
#'     lawschool = df.att$lawschool - 1
#' )
#'
#' # calculate joint entropies
#' J <- joint_entropy(df.att.ed)
#' # joint entropy matrix
#' J$matrix
#' # frequency distribution of joint entropy values
#' J$freq
#' @export
#'
#'

joint_entropy <- function(dat, dec = 3) {
    varname_orig <- colnames(dat)
    varname_new <- sprintf("V%d", seq_len(length(dat)))
    names(dat) <- varname_new

    J <- matrix(0, nrow = ncol(dat), ncol = ncol(dat))
    colnames(J) <- colnames(dat)
    rownames(J) <- colnames(dat)

    # get the bivariate entropies H
    H <- entropy_bivar(dat)

    # joint entropies after calculation of H matrix
    for (x in seq_len(ncol(H))) {
        for (y in (x):ncol(H)) {
            J[x, y] <- H[x, x] + H[y, y] - H[x, y]
        }
    }

    # given input argument dec giving precision, round J
    J <- round(J, dec)
    colnames(J) <- varname_orig
    rownames(J) <- varname_orig
    J[lower.tri(J)] <- NA


    # frequency distribution of the joint entropy values
    FrqJ <- as.data.frame(table(round(J[upper.tri(J, diag = FALSE)], dec)))
    FrqJ <- FrqJ[order(FrqJ$Var1, decreasing = TRUE), ]
    FrqJ$CumFreq <- cumsum(FrqJ$Freq)
    names(FrqJ)[names(FrqJ) == "Var1"] <- "j"
    names(FrqJ)[names(FrqJ) == "Freq"] <- " #(J = j)"
    names(FrqJ)[names(FrqJ) == "CumFreq"] <- "#(J >= j)"
    row.names(FrqJ) <- NULL

    listout <- list("matrix" = J, "freq" = FrqJ)
    return(listout)
}

test_that("joint entropies work", {
  data(lawdata)
  df.att <- lawdata[[4]]

  df.att.ed <- data.frame(
    status   = df.att$status,
    gender   = df.att$gender,
    office   = df.att$office-1,
    years    = ifelse(df.att$years<=3,0,
                      ifelse(df.att$years<=13,1,2)),
    age      = ifelse(df.att$age<=35,0,
                      ifelse(df.att$age<=45,1,2)),
    practice = df.att$practice,
    lawschool= df.att$lawschool-1)

  H.joint <- joint_entropy(df.att.ed)
  result <- c(status = 1, gender = 0.817, office = 1.125, years = 1.585,
              age = 1.585, practice = 0.983, lawschool = 1.533)
  expect_identical(diag(H.joint$matrix),result)
})

#' @title Divergence Tests of Goodness of Fit
#' @description Performs divergence-based goodness-of-fit tests for discrete data,
#' including tests of uniformity, pairwise independence, conditional independence,
#' and nested model comparisons.
#' @param dat dataframe with rows as observations and columns as variables.
#' Variables must be categorical with finite range spaces.
#' @param var_uniform character name of a variable in \code{dat} to test for uniformity.
#' @param var1 character name of the first variable.
#' @param var2 character name of the second variable.
#' @param var_cond optional character vector of conditioning variables.
#' @param model_full list containing \code{D} and \code{df} for the full model.
#' @param model_reduced list containing \code{D} and \code{df} for the reduced model.
#' @param alpha significance level. Default is 0.05.
#' @param dec number of decimals for rounding. Default is 3.
#' @param use_approx_cv logical; if \code{TRUE}, uses the approximate critical value
#' \code{df + sqrt(8 * df)}. If \code{FALSE}, uses the chi-square quantile.
#' @return Dataframe with test type, divergence \emph{D}, chi-square statistic,
#' degrees of freedom, critical value, and decision.
#' @details
#' The function implements four types of tests:
#'
#' \strong{1. Uniformity}
#' \deqn{D = \log r_X - H(X)}
#'
#' \strong{2. Pairwise Independence}
#' \deqn{D = H(X) + H(Y) - H(X,Y)}
#'
#' \strong{3. Conditional Independence}
#' \deqn{D = H(X,Z) + H(Y,Z) - H(Z) - H(X,Y,Z)}
#'
#' where \emph{Z} may also represent a vector of conditioning variables.
#'
#' \strong{4. Nested Model Comparison}
#' \deqn{D = D_{reduced} - D_{full}}
#'
#' The test statistic is
#' \deqn{2nD\log(2),}
#' since entropies are computed using base 2 logarithms.
#'
#' Smaller divergence values indicate better model fit.
#' @author Termeh Shafie
#' @seealso \code{\link{joint_entropy}}, \code{\link{entropy_trivar}}
#' @references Frank, O., & Shafie, T. (2016). Multivariate entropy analysis of network data.
#' \emph{Bulletin of Sociological Methodology/Bulletin de Méthodologie Sociologique}, 129(1), 45-63.
#' @examples
#' data(lawdata)
#' df_att <- lawdata[[4]]
#'
#' att_var <- data.frame(
#'   status    = df_att$status - 1,
#'   gender    = df_att$gender,
#'   office    = df_att$office - 1,
#'   years     = ifelse(df_att$years <= 3, 0,
#'                 ifelse(df_att$years <= 13, 1, 2)),
#'   age       = ifelse(df_att$age <= 35, 0,
#'                 ifelse(df_att$age <= 45, 1, 2)),
#'   practice  = df_att$practice,
#'   lawschool = df_att$lawschool - 1
#' )
#'
#' ## 1. Test uniformity
#' div_gof(att_var, var_uniform = "gender")
#'
#' ## 2. Test pairwise independence
#' div_gof(att_var, var1 = "status", var2 = "gender")
#'
#' ## 3. Test conditional independence
#'
#' ## (a) Conditional independence given a single variable
#' div_gof(att_var,
#'         var1 = "status",
#'         var2 = "gender",
#'         var_cond = "years")
#'
#' ## (b) Conditional independence given multiple variables
#' div_gof(att_var,
#'         var1 = "status",
#'         var2 = "gender",
#'         var_cond = c("years", "age"))
#'
#' ## 4. Nested model comparison
#' ## Compare reduced models against the saturated empirical model.
#' ## The saturated model has divergence D = 0 and df = 0.
#' m_full <- list(D = 0, df = 0)
#'
#' ## (a) Pairwise independence model
#' m_reduced <- div_gof(att_var,
#'                     var1 = "status",
#'                     var2 = "gender")
#'
#' div_gof(att_var,
#'         model_full = m_full,
#'         model_reduced = list(D = m_reduced$D, df = m_reduced$df))
#'
#' ## (b) Conditional independence model
#' m_reduced <- div_gof(att_var,
#'                     var1 = "status",
#'                     var2 = "gender",
#'                     var_cond = "years")
#'
#' div_gof(att_var,
#'         model_full = m_full,
#'         model_reduced = list(D = m_reduced$D, df = m_reduced$df))
#'
#' ## 5. Nested comparison against the saturated empirical model
#' m_full <- list(D = 0, df = 0)
#'
#' m_reduced <- div_gof(att_var,
#'                     var1 = "status",
#'                     var2 = "gender")
#'
#' div_gof(att_var,
#'         model_full = m_full,
#'         model_reduced = list(D = m_reduced$D, df = m_reduced$df))
#' @export

div_gof <- function(dat,
                    var_uniform = NULL,
                    var1 = NULL,
                    var2 = NULL,
                    var_cond = NULL,
                    model_full = NULL,
                    model_reduced = NULL,
                    alpha = 0.05,
                    dec = 3,
                    use_approx_cv = TRUE) {

  entropy_set <- function(dat, vars) {
    tab <- table(dat[, vars, drop = FALSE])
    p <- as.vector(tab) / sum(tab)
    p <- p[p > 0]
    sum(p * log2(1 / p))
  }

  n_levels <- function(x) length(unique(x))

  make_output <- function(test, D, df_chi2) {
    if (df_chi2 <= 0) {
      stop("Degrees of freedom must be positive.")
    }

    chi2_stat <- 2 * nrow(dat) * D * log(2)

    crit_val <- if (use_approx_cv) {
      df_chi2 + sqrt(8 * df_chi2)
    } else {
      stats::qchisq(1 - alpha, df = df_chi2)
    }

    decision <- if (chi2_stat > crit_val) {
      "reject"
    } else {
      "cannot reject"
    }

    data.frame(
      test = test,
      D = round(D, dec),
      chi2 = round(chi2_stat, dec),
      df = df_chi2,
      critical_value = round(crit_val, dec),
      decision = decision
    )
  }

  if (!is.null(var_uniform) &&
      is.null(var1) &&
      is.null(var2) &&
      is.null(var_cond) &&
      is.null(model_full) &&
      is.null(model_reduced)) {

    r_x <- n_levels(dat[[var_uniform]])
    D <- log2(r_x) - entropy_set(dat, var_uniform)
    df_chi2 <- r_x - 1

    return(make_output(
      test = paste0("uniformity: ", var_uniform),
      D = D,
      df_chi2 = df_chi2
    ))
  }

  if (!is.null(var1) &&
      !is.null(var2) &&
      is.null(var_cond) &&
      is.null(model_full) &&
      is.null(model_reduced)) {

    r_x <- n_levels(dat[[var1]])
    r_y <- n_levels(dat[[var2]])

    D <- entropy_set(dat, var1) +
      entropy_set(dat, var2) -
      entropy_set(dat, c(var1, var2))

    df_chi2 <- (r_x - 1) * (r_y - 1)

    return(make_output(
      test = paste0(var1, " independent of ", var2),
      D = D,
      df_chi2 = df_chi2
    ))
  }

  if (!is.null(var1) &&
      !is.null(var2) &&
      !is.null(var_cond) &&
      is.null(model_full) &&
      is.null(model_reduced)) {

    r_x <- n_levels(dat[[var1]])
    r_y <- n_levels(dat[[var2]])
    r_cond <- prod(vapply(dat[var_cond], n_levels, numeric(1)))

    D <- entropy_set(dat, c(var1, var_cond)) +
      entropy_set(dat, c(var2, var_cond)) -
      entropy_set(dat, var_cond) -
      entropy_set(dat, c(var1, var2, var_cond))

    df_chi2 <- (r_x - 1) * (r_y - 1) * r_cond

    return(make_output(
      test = paste0(
        var1, " independent of ", var2,
        " given ", paste(var_cond, collapse = ", ")
      ),
      D = D,
      df_chi2 = df_chi2
    ))
  }

  if (!is.null(model_full) && !is.null(model_reduced)) {
    D <- model_reduced$D - model_full$D
    df_chi2 <- model_reduced$df - model_full$df

    if (df_chi2 <= 0) {
      stop("Invalid nested comparison: `model_reduced$df` must be larger than `model_full$df`.")
    }

    if (D < 0) {
      stop("Invalid nested comparison: `model_reduced$D` must be at least as large as `model_full$D`.")
    }

    return(make_output(
      test = "nested model comparison",
      D = D,
      df_chi2 = df_chi2
    ))
  }

  stop(
    "Specify one of: `var_uniform`, (`var1`, `var2`), ",
    "(`var1`, `var2`, `var_cond`), or (`model_full`, `model_reduced`)."
  )
}

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(out.width = "100%",
  cache = FALSE
)

## ----load_library, eval=TRUE--------------------------------------------------
library(netropy)

## ----load_data, eval=TRUE, include=FALSE--------------------------------------
data(lawdata) 
adj.advice <- lawdata[[1]]
adj.friend <- lawdata[[2]]
adj.cowork <-lawdata[[3]]
df.att <- lawdata[[4]]

## ----edit_data, eval=TRUE, include=FALSE, results ='hide'---------------------
att.var <-
  data.frame(
    status   = df.att$status-1,
    gender   = df.att$gender,
    office   = df.att$office-1,
    years    = ifelse(df.att$years<=3,0,
                      ifelse(df.att$years<=13,1,2)),
    age      = ifelse(df.att$age<=35,0,
                      ifelse(df.att$age<=45,1,2)),
    practice = df.att$practice,
    lawschool= df.att$lawschool-1
    )
dyad.status    <- get_dyad_var(att.var$status, type = 'att')
dyad.gender    <- get_dyad_var(att.var$gender, type = 'att')
dyad.office    <- get_dyad_var(att.var$office, type = 'att')
dyad.years     <- get_dyad_var(att.var$years, type = 'att')
dyad.age       <- get_dyad_var(att.var$age, type = 'att')
dyad.practice  <- get_dyad_var(att.var$practice, type = 'att')
dyad.lawschool <- get_dyad_var(att.var$lawschool, type = 'att')
dyad.cwk    <- get_dyad_var(adj.cowork, type = 'tie')
dyad.adv    <- get_dyad_var(adj.advice, type = 'tie')
dyad.frn    <- get_dyad_var(adj.friend, type = 'tie')
dyad.var <-
  data.frame(cbind(status    = dyad.status$var,
                  gender    = dyad.gender$var,
                  office    = dyad.office$var,
                  years     = dyad.years$var,
                  age       = dyad.age$var,
                  practice  = dyad.practice$var,
                  lawschool = dyad.lawschool$var,
                  cowork    = dyad.cwk$var,
                  advice    = dyad.adv$var,
                  friend    = dyad.frn$var)
                  )

## ----show_data, eval=TRUE, include=TRUE, results ='markup'--------------------
head(dyad.var)

## ----biv_ent, eval=TRUE, include=TRUE, results ='markup'----------------------
entropy_bivar(dyad.var)

## ----red, eval=TRUE, include=TRUE, results ='markup'--------------------------
redundancy(dyad.var)

## ----edit_data2, eval=TRUE, include=FALSE, results ='hide'--------------------
att.var <-
  data.frame(
    senior   = df.att$senior,
    status   = df.att$status-1,
    gender   = df.att$gender,
    office   = df.att$office-1,
    years    = ifelse(df.att$years<=3,0,
                      ifelse(df.att$years<=13,1,2)),
    age      = ifelse(df.att$age<=35,0,
                      ifelse(df.att$age<=45,1,2)),
    practice = df.att$practice,
    lawschool= df.att$lawschool-1
    )

## ----show_data2, eval=TRUE, include=TRUE, results ='markup'-------------------
head(att.var)

## ----red2, eval=TRUE, include=TRUE, results ='markup'-------------------------
redundancy(att.var)

## ----biv_ent2, eval=TRUE, include=TRUE, results ='markup'---------------------
entropy_bivar(att.var)

## ----triv_ent, eval=TRUE, include=TRUE, results ='markup'---------------------
entropy_trivar(dyad.var)

## ----tetrav_ent, eval=TRUE, include=TRUE, results='markup'--------------------
entropy_tetravar(
  dat = dyad.var[, c("gender", "years", "age", "office", "practice")]
)

#' @title Prediction Power
#' @description
#' Computes prediction power when pairs of variables in a given dataframe are used
#' to predict a third variable from the same dataframe. The prediction strength is measured by
#' expected conditional entropies.
#' @param dat dataframe with rows as observations and columns as variables.
#' Variables must all be observed or transformed categorical with finite range spaces.
#' @param var character string representing the variable in
#' dataframe \code{dat} to be predicted by pairs of other variables in the dataframe \code{dat}.
#' @return Upper triangular matrix giving the expected conditional entropies of pairs of variables
#' given as rows and columns of the matrix. The diagonal gives \emph{EH(Z|X) = H(X,Z) - H(X)}, that is
#' when only one variable is used to predict \code{var}. Note that \code{NA}'s are in the entire
#' row and column representing the variable being predicted.
#' @details The expected conditional entropy given by\cr
#'
#' \emph{EH(Z|X,Y) = H(X,Y,Z) - H(X, Y)} \cr
#'
#' measures the prediction uncertainty when pairs of variables \emph{X} and \emph{Y}
#' are used to predict variable \emph{Z}.
#' The lower the value of \emph{EH} given different pairs of variables \emph{X} and \emph{Y},
#' the stronger is the prediction of \emph{Z}.
#' @author Termeh Shafie
#' @seealso \code{\link{entropy_trivar}}, \code{\link{entropy_bivar}}
#' @references Frank, O., & Shafie, T. (2016). Multivariate entropy analysis of network data.
#' \emph{Bulletin of Sociological Methodology/Bulletin de Méthodologie Sociologique}, 129(1), 45-63.
#' @examples
#' # use internal data set
#' data(lawdata)
#' df.att <- lawdata[[4]]
#'
#' # three steps of data editing:
#' # 1. categorize variables 'years' and 'age' based on
#' # approximately three equally size groups (values based on cdf)
#' # 2. make sure all outcomes start from the value 0 (optional)
#' # 3. remove variable 'senior' as it consists of only unique values (thus redundant)
#' df.att.ed <- data.frame(
#'     status = df.att$status,
#'     gender = df.att$gender,
#'     office = df.att$office - 1,
#'     years = ifelse(df.att$years <= 3, 0,
#'         ifelse(df.att$years <= 13, 1, 2)
#'     ),
#'     age = ifelse(df.att$age <= 35, 0,
#'         ifelse(df.att$age <= 45, 1, 2)
#'     ),
#'     practice = df.att$practice,
#'     lawschool = df.att$lawschool - 1
#' )
#'
#' # power of predicting 'status' using pairs of other variables
#' prediction_power("status", df.att.ed)

#' @export

prediction_power <- function(var, dat) {
    z <- which(names(dat) == var)

    varname_orig <- colnames(dat)
    varname_new <- sprintf("V%d", seq_len(ncol(dat)))
    names(dat) <- varname_new

    H2 <- entropy_bivar(dat)
    H3 <- entropy_trivar(dat)


    H3$X <- as.numeric(gsub("V", "", H3$X))
    H3$Y <- as.numeric(gsub("V", "", H3$Y))
    H3$Z <- as.numeric(gsub("V", "", H3$Z))

    H3 <- as.matrix(H3)
    dimE <- max(H3[, 1:3])

    idz <- which(apply(H3[, 1:3], 1, function(x) any(x == z)))
    EHZXY <- matrix(NA, dimE, dimE)
    for (xy in idz) {
        idxy <- sort(setdiff(H3[xy, 1:3], z))
        x <- idxy[1]
        y <- idxy[2]
        x.coord <- which(paste0("V", x) == colnames(H2))
        y.coord <- which(paste0("V", y) == colnames(H2))
        z.coord <- which(paste0("V", z) == colnames(H2))
        EHZXY[x, y] <- H3[xy, 4] - H2[x.coord, y.coord]
    }

    # add the diagonal
    z.coord <- which(paste0("V", z) == colnames(H2))
    H2[lower.tri(H2)] <- H2[upper.tri(H2)]
    for (x in seq_len(nrow(H2))) {
        if (x != z) {
            EHZXY[x, x] <- H2[x, z.coord] - H2[z.coord, z.coord]
        }
    }

    colnames(EHZXY) <- varname_orig
    rownames(EHZXY) <- varname_orig
    EHZXY <- as.matrix(EHZXY)
    EHZXY <- round(EHZXY, 3)

    return(EHZXY)
}

test_that("prediction power works", {
  data(lawdata)
  df.att <- lawdata[[4]]

  df.att.ed <- data.frame(
    status   = df.att$status,
    gender   = df.att$gender,
    office   = df.att$office-1,
    years    = ifelse(df.att$years<=3,0,
                      ifelse(df.att$years<=13,1,2)),
    age      = ifelse(df.att$age<=35,0,
                      ifelse(df.att$age<=45,1,2)),
    practice = df.att$practice,
    lawschool= df.att$lawschool-1)
    ppower <- prediction_power('status', df.att.ed)
    result <- c(status = NA, gender = 0.695, office = 1.084, years = 0.927,
                age = 1.007, practice = 1.226, lawschool = 1.693)
    expect_identical(diag(ppower),result)

})

#' @title Get Dyad Variables
#' @description Transforms vertex variables or observed directed/undirected ties
#' into dyad variables.
#' @param var variable vector (actor attribute) or adjacency matrix (ties)
#' to be transformed to a dyad variable.
#' @param type either 'att' for actor attribute (default) or 'tie' for relations.
#' @return Dataframe with three columns:
#' first two columns show the vertex pairs \code{u} and \code{v} where \code{u<v} ,
#' and the third column gives the value of the transformed dyadic variable \code{var}.
#' @details Dyad variables are given as pairs of incident vertex variables
#' or actor attributes. Here, unique pairs of original attribute values
#' constitute the outcome space. Note that the actor  attributes need
#' to be categorical with finite range spaces. For example, binary
#' attribute yields outcome space (0,0), (0,1), (1,0), (1,1) coded as (0),(1),(2),(3).
#' Warning message is shown if actor attribute has too many unique outcomes
#' as it will yield too many possible outcomes once converted in to a dyad variable.
#' \cr
#'
#' For directed relations, pairs of indicators from the adjacency matrix constitute
#' the four outcomes representing possible combinations of sending and receiving ties:
#' (0,0), (0,1), (1,0), (1,1) coded as (0),(1),(2),(3).
#' \cr
#'
#' For undirected relations, an indicator variable which is directly read from the
#' adjacency matrix represents the dyadic variable.
#' \cr
#'
#' @author Termeh Shafie
#' @seealso \code{\link{get_triad_var}}
#' @references Frank, O., & Shafie, T. (2016). Multivariate entropy analysis of network data.
#' \emph{Bulletin of Sociological Methodology/Bulletin de Méthodologie Sociologique}, 129(1), 45-63.
#' @examples
#' # use internal data set
#' data(lawdata)
#' adj.advice <- lawdata[[1]]
#' adj.cowork <- lawdata[[3]]
#' df.att <- lawdata[[4]]
#'
#' # three steps of data editing of attribute dataframe:
#' # 1. categorize variables 'years' and 'age' based on
#' # approximately three equally size groups (values based on cdf)
#' # 2. make sure all outcomes start from the value 0 (optional)
#' # 3. remove variable 'senior' as it consists of only unique values (thus redundant)
#' df.att.ed <- data.frame(
#'     status = df.att$status,
#'     gender = df.att$gender,
#'     office = df.att$office - 1,
#'     years = ifelse(df.att$years <= 3, 0,
#'         ifelse(df.att$years <= 13, 1, 2)
#'     ),
#'     age = ifelse(df.att$age <= 35, 0,
#'         ifelse(df.att$age <= 45, 1, 2)
#'     ),
#'     practice = df.att$practice,
#'     lawschool = df.att$lawschool - 1
#' )
#'
#' # actor attribute converted to dyad variable
#' dyad.gend <- get_dyad_var(df.att.ed$gender, "att")
#'
#' # directed tie converted to dyad variable
#' dyad.adv <- get_dyad_var(adj.advice, "tie")
#'
#' # undirected tie converted to dyad variable
#' dyad.cwk <- get_dyad_var(adj.cowork, "tie")
#' @export

get_dyad_var <- function(var, type = "att") {
    if (type == "att") {
        n <- length(var)
        if (length(unique(var)) >= 7) {
            warning("consider categorizing variable as it yields too many dyadic outcomes")
        }
        var.matching <- outer(c(min(var):max(var)), c(min(var):max(var)), paste)
        dyad <- expand.grid(u = 1:n, v = 1:n)
        dyad$var <- match(paste(var[dyad$u], var[dyad$v]), var.matching) - 1
        dyad <- dyad[dyad$u < dyad$v, ]
    } else if (type == "tie") {
        n <- dim(var)[1]
        if (isSymmetric(var) == TRUE) {
            dyad <- data.frame(
                u = rep(1:n, n), v = rep(1:n, each = n),
                var = c(var)
            )
            dyad <- dyad[dyad$u < dyad$v, ]
            message("two outcomes based on an indicator variable for the undirected relation is created")
        } else if (isSymmetric(var) == FALSE) {
            var.matching <- outer(c(0, 1), c(0, 1), paste)
            dyad <- data.frame(
                u = rep(1:n, n), v = rep(1:n, each = n),
                var = paste(c(t(var)), c(var))
            )
            dyad$var <- match(dyad$var, var.matching) - 1
            dyad <- dyad[dyad$u < dyad$v, ]
            message("four outcomes based on pairs of indicators for the directed relation is created")
        }
    }

    return(dyad)
}

#' @title Tetravariate Entropy
#' @description Computes tetravariate entropies, expected conditional entropies,
#' and expected conditional joint entropies for all quadruples of variables in a
#' multivariate discrete data set.
#' @param dat dataframe with rows as observations and columns as variables.
#' Variables must be categorical with finite range spaces.
#' @param dec number of decimals used for rounding the entropy values.
#' Default is 2.
#' @return A dataframe with one row for each ordered decomposition of four
#' variables into predictors and conditioning variables. The columns are:
#' \item{X}{first variable in the pair of interest.}
#' \item{Y}{second variable in the pair of interest.}
#' \item{Z}{first conditioning variable.}
#' \item{U}{second conditioning variable.}
#' \item{H_XYZU}{tetravariate entropy \emph{H(X,Y,Z,U)}.}
#' \item{EH_U_XYZ}{expected conditional entropy \emph{EH(U|X,Y,Z)}.}
#' \item{EH_Z_XYU}{expected conditional entropy \emph{EH(Z|X,Y,U)}.}
#' \item{EJ_XY_ZU}{expected conditional joint entropy \emph{EJ(X,Y|Z,U)}.}
#' @details For four variables \emph{X}, \emph{Y}, \emph{Z}, and \emph{U}, the
#' tetravariate entropy is denoted \emph{H(X,Y,Z,U)}. The expected conditional
#' entropies are computed as
#' \deqn{EH(U|X,Y,Z) = H(X,Y,Z,U) - H(X,Y,Z)}
#' and
#' \deqn{EH(Z|X,Y,U) = H(X,Y,Z,U) - H(X,Y,U).}
#' The expected conditional joint entropy is computed as
#' \deqn{EJ(X,Y|Z,U) = H(X,Z,U) + H(Y,Z,U) - H(Z,U) - H(X,Y,Z,U).}
#' This quantity measures deviation from conditional independence of the form
#' \eqn{X \perp Y \,\vert\, Z, U}{X is independent of Y given Z, U}.
#' Smaller values indicate weaker conditional dependence.
#' @author Termeh Shafie
#' @seealso \code{\link{entropy_trivar}}, \code{\link{entropy_bivar}},
#' \code{\link{prediction_power}}
#' @references Frank, O., & Shafie, T. (2016). Multivariate entropy analysis of
#' network data. \emph{Bulletin of Sociological Methodology/Bulletin de
#' Méthodologie Sociologique}, 129(1), 45-63.
#' @examples
#' # use internal data set
#' data(lawdata)
#'
#' # extract node attributes
#' df_att <- lawdata[[4]]
#'
#' # data editing:
#' # 1. discretize 'years' and 'age' into three approximately balanced groups
#' # 2. recode selected variables so categories start at 0
#' att_var <- data.frame(
#'   status    = df_att$status - 1,
#'   gender    = df_att$gender,
#'   office    = df_att$office - 1,
#'   years     = ifelse(df_att$years <= 3, 0,
#'                 ifelse(df_att$years <= 13, 1, 2)),
#'   age       = ifelse(df_att$age <= 35, 0,
#'                 ifelse(df_att$age <= 45, 1, 2)),
#'   practice  = df_att$practice,
#'   lawschool = df_att$lawschool - 1
#' )
#'
#' # compute tetravariate entropy quantities for five selected variables
#' entropy_tetravar(
#'   dat = att_var[, c("gender", "years", "age", "office", "practice")]
#' )
#'
#' @export

entropy_tetravar <- function(dat, dec = 2) {
  entropy_emp <- function(x) {
    tab <- table(x)
    p <- as.vector(tab) / sum(tab)
    p <- p[p > 0]
    sum(p * log2(1 / p))
  }

  vars <- colnames(dat)
  n_out <- choose(length(vars), 4) * 6

  cache <- new.env(hash = TRUE, parent = emptyenv())
  H <- function(cols) {
    key <- paste(sort(cols), collapse = "\r")
    val <- cache[[key]]
    if (is.null(val)) {
      val <- entropy_emp(dat[cols])
      cache[[key]] <- val
    }
    val
  }

  X <- character(n_out)
  Y <- character(n_out)
  Z <- character(n_out)
  U <- character(n_out)
  H_XYZU <- numeric(n_out)
  EH_U_XYZ <- numeric(n_out)
  EH_Z_XYU <- numeric(n_out)
  EJ_XY_ZU <- numeric(n_out)

  k <- 1
  for (quad in utils::combn(vars, 4, simplify = FALSE)) {
    h_xyzu <- H(quad)
    xy_pairs <- utils::combn(quad, 2, simplify = FALSE)

    for (xy in xy_pairs) {
      zu <- setdiff(quad, xy)

      x <- xy[1]
      y <- xy[2]
      z <- zu[1]
      u <- zu[2]

      h_xyz <- H(c(x, y, z))
      h_xyu <- H(c(x, y, u))
      h_xzu <- H(c(x, z, u))
      h_yzu <- H(c(y, z, u))
      h_zu <- H(c(z, u))

      X[k] <- x
      Y[k] <- y
      Z[k] <- z
      U[k] <- u
      H_XYZU[k] <- h_xyzu
      EH_U_XYZ[k] <- h_xyzu - h_xyz
      EH_Z_XYU[k] <- h_xyzu - h_xyu
      EJ_XY_ZU[k] <- h_xzu + h_yzu - h_zu - h_xyzu

      k <- k + 1
    }
  }

  data.frame(
    X = X,
    Y = Y,
    Z = Z,
    U = U,
    H_XYZU = round(H_XYZU, dec),
    EH_U_XYZ = round(EH_U_XYZ, dec),
    EH_Z_XYU = round(EH_Z_XYU, dec),
    EJ_XY_ZU = round(EJ_XY_ZU, dec),
    stringsAsFactors = FALSE
  )
}

test_that("tetravariate entropy works", {
  data(lawdata)
  df.att <- lawdata[[4]]

  att_var <- data.frame(
    status    = df.att$status - 1,
    gender    = df.att$gender,
    office    = df.att$office - 1,
    years     = ifelse(df.att$years <= 3, 0,
                  ifelse(df.att$years <= 13, 1, 2)),
    age       = ifelse(df.att$age <= 35, 0,
                  ifelse(df.att$age <= 45, 1, 2)),
    practice  = df.att$practice,
    lawschool = df.att$lawschool - 1
  )

  h4 <- entropy_tetravar(
    att_var[, c("gender", "years", "age", "office", "practice")]
  )

  expect_identical(
    colnames(h4),
    c("X", "Y", "Z", "U", "H_XYZU", "EH_U_XYZ", "EH_Z_XYU", "EJ_XY_ZU")
  )
  expect_identical(nrow(h4), 30L)

  expect_identical(
    h4$H_XYZU[1:6],
    c(4.22, 4.22, 4.22, 4.22, 4.22, 4.22)
  )
  expect_identical(
    h4$EH_U_XYZ[1:6],
    c(0.88, 0.88, 0.92, 0.88, 0.92, 0.81)
  )
  expect_identical(
    h4$EH_Z_XYU[1:6],
    c(0.92, 0.81, 0.81, 0.46, 0.46, 0.46)
  )
  expect_identical(
    h4$EJ_XY_ZU[1:6],
    c(0.29, 0.14, 0.12, 0.56, 0.14, 0.18)
  )
})

test_that("tetravariate entropy respects dec argument", {
  data(lawdata)
  df.att <- lawdata[[4]]

  att_var <- data.frame(
    gender   = df.att$gender,
    office   = df.att$office - 1,
    practice = df.att$practice,
    status   = df.att$status - 1
  )

  h4_2 <- entropy_tetravar(att_var, dec = 2)
  h4_4 <- entropy_tetravar(att_var, dec = 4)

  expect_identical(h4_2$H_XYZU, round(h4_4$H_XYZU, 2))
  expect_identical(h4_2$EJ_XY_ZU, round(h4_4$EJ_XY_ZU, 2))
})

#' @title Get Triad Variables
#' @description Transforms vertex variables or observed directed/undirected ties
#' into triad variables.
#' @param var variable vector (actor attribute) or adjacency matrix (ties)
#' to be transformed to a triad variable.
#' @param type either 'att' for actor attribute (default) or 'tie' for relations.
#' @return Dataframe with four columns:
#' first three columns show the vertex triad \code{u}, \code{v}, \code{w} ,
#' and the fourth column gives the value of the transformed triadic variable \code{var}.
#' @details For actor attributes, unique triples of original attribute values
#' constitute the outcome space. Note that the actor
#' attributes need to be categorical with finite range spaces.
#' For example, binary attributes have 8 possible triadic outcomes
#' (0,0,0),(1,0,0),(0,1,0),(1,1,0),(0,0,1),(1,0,1),(0,1,1),(1,1,1)
#' which are coded 0-7.
#' Warning message is shown if actor attribute has too many unique outcomes
#' as it will yield too many possible outcomes once converted in to a triad variable.
#' \cr
#'
#' For directed relations, a sequence of indicators of length 6 created from the adjacency matrix
#' constitutes the 64 outcomes representing possible combinations of sending and receiving ties.
#' \cr
#'
#' For undirected relations, triples of indicators are created from the adjacency matrix.
#' \cr
#'
#'
#' @author Termeh Shafie
#' @seealso \code{\link{get_dyad_var}}
#' @references Frank, O., & Shafie, T. (2016). Multivariate entropy analysis of network data.
#' \emph{Bulletin of Sociological Methodology/Bulletin de Méthodologie Sociologique}, 129(1), 45-63.
#' @examples
#' # use internal data set
#' data(lawdata)
#' adj.advice <- lawdata[[1]]
#' adj.cowork <- lawdata[[3]]
#' df.att <- lawdata[[4]]
#'
#' # three steps of data editing:
#' # 1. categorize variables 'years' and 'age' based on
#' # approximately three equally size groups (values based on cdf)
#' # 2. make sure all outcomes start from the value 0 (optional)
#' # 3. remove variable 'senior' as it consists of only unique values (thus redundant)
#' df.att.ed <- data.frame(
#'     status = df.att$status,
#'     gender = df.att$gender,
#'     office = df.att$office - 1,
#'     years = ifelse(df.att$years <= 3, 0,
#'         ifelse(df.att$years <= 13, 1, 2)
#'     ),
#'     age = ifelse(df.att$age <= 35, 0,
#'         ifelse(df.att$age <= 45, 1, 2)
#'     ),
#'     practice = df.att$practice,
#'     lawschool = df.att$lawschool - 1
#' )
#'
#' # actor attribute converted to triad variable
#' triad.gend <- get_triad_var(df.att.ed$gender, "att")
#'
#' # directed tie converted to triad variable
#' triad.adv <- get_triad_var(adj.advice, type = "tie")
#'
#' # undirected tie converted to triad variable
#' triad.cwk <- get_triad_var(adj.cowork, type = "tie")
#' @export

get_triad_var <- function(var, type = "att") {
    if (type == "att") {
        n <- length(var)
        triads <- t(utils::combn(1:n, 3))
        triads <- cbind(triads, var[triads[, 1]], var[triads[, 2]], var[triads[, 3]])
        triad.var <- data.frame(
            u = triads[, 1], v = triads[, 2], w = triads[, 3],
            u.var = triads[, 4], v.var = triads[, 5], w.var = triads[, 6]
        )
        triad.var$uvw <- paste(
            triad.var$u.var,
            triad.var$v.var,
            triad.var$w.var
        )
        var.matching <- apply(expand.grid(
            min(var):max(var),
            min(var):max(var),
            min(var):max(var)
        ), 1, paste, collapse = " ")
        triad.var$var <- match(triad.var$uvw, var.matching) - 1
        if (length(unique(var)) >= 7) {
            warning("consider categorizing variable as it yields too many dyadic outcomes")
        }
    } else if (type == "tie") {
        n <- dim(var)[1]
        if (isSymmetric(var) == TRUE) {
            triads <- t(utils::combn(1:n, 3))
            triads <- cbind(
                triads,
                var[triads[, c(1, 2)]],
                var[triads[, c(1, 3)]],
                var[triads[, c(2, 3)]]
            )
            colnames(triads) <- c(
                "u", "v", "w",
                "uv", "uw", "vw"
            )
            triad.var <- as.data.frame(triads)
            triad.var$uv.uw.vw <- apply(triad.var[, 4:6], 1,
                paste,
                collapse = " "
            )
            grid <- expand.grid(0:1, 0:1, 0:1)
            var.matching <- apply(
                grid, 1,
                function(x) paste(x, collapse = " ")
            )
            triad.var$var <- match(triad.var$uv.uw.vw, var.matching) - 1
            message("8 outcomes based on triples of indicators for the undirected relation are created")
        } else if (isSymmetric(var) == FALSE) {
            triads <- t(utils::combn(1:n, 3))
            triads <- cbind(
                triads, var[triads[, c(1, 2)]],
                var[triads[, c(2, 1)]],
                var[triads[, c(1, 3)]],
                var[triads[, c(3, 1)]],
                var[triads[, c(2, 3)]],
                var[triads[, c(3, 2)]]
            )
            colnames(triads) <- c(
                "u", "v", "w",
                "uv", "vu", "uw", "wu", "vw", "wv"
            )
            triad.var <- as.data.frame(triads)
            triad.var$seq <- apply(triad.var[, 4:9], 1,
                paste,
                collapse = " "
            )
            grid <- expand.grid(0:1, 0:1, 0:1, 0:1, 0:1, 0:1)
            var.matching <- apply(
                grid, 1,
                function(x) paste(x, collapse = " ")
            )
            triad.var$var <- match(triad.var$seq, var.matching) - 1
            message("64 outcomes based on a sequence of 6 indicators for the directed relation are created")
        }
    }

    out.var <- as.data.frame(cbind(triad.var$u, triad.var$v, triad.var$w, triad.var$var))
    names(out.var) <- c("u", "v", "w", "var")
    return(out.var)
}

#' @title Association Graphs
#' @description Draws association graphs (graphical models) based on joint entropy values
#' to detect and visualize different dependence structures among the variables in the dataframe.
#' @param dat dataframe with rows as observations and columns as variables. Variables must all be observed or transformed categorical with finite range spaces.
#' @param cutoff the cutoff point for the edges to be drawn based on joint entropies. Default is 0 and draws all edges.
#' @return A ggraph object with nodes representing all variables in \code{dat} and edges
#' representing (the strength of) associations between them based on joint entropies.
#' @details Draws association graphs based on given thresholds of joint entropy values
#' between pairs of variables represented as nodes. Thickness of edges between pairs of nodes/variables
#' indicates the strength of dependence between them. Isolated nodes are completely
#' independent and paths through certain nodes/variables indicate conditional dependencies.
#' @author Termeh Shafie
#' @seealso \code{\link{joint_entropy}}
#' @references Frank, O., & Shafie, T. (2016). Multivariate entropy analysis of network data.
#' \emph{Bulletin of Sociological Methodology/Bulletin de Méthodologie Sociologique}, 129(1), 45-63.
#' @examples
#' library(ggraph)
#' # use internal data set
#' data(lawdata)
#' df.att <- lawdata[[4]]
#'
#' # three steps of data editing:
#' # 1. categorize variables 'years' and 'age' based on
#' # approximately three equally size groups (values based on cdf)
#' # 2. make sure all outcomes start from the value 0 (optional)
#' # 3. remove variable 'senior' as it consists of only unique values (thus redundant)
#' df.att.ed <- data.frame(
#'     status = df.att$status,
#'     gender = df.att$gender,
#'     office = df.att$office - 1,
#'     years = ifelse(df.att$years <= 3, 0,
#'         ifelse(df.att$years <= 13, 1, 2)
#'     ),
#'     age = ifelse(df.att$age <= 35, 0,
#'         ifelse(df.att$age <= 45, 1, 2)
#'     ),
#'     practice = df.att$practice,
#'     lawschool = df.att$lawschool - 1
#' )
#'
#' # association graph based on cutoff 0.15
#' assoc_graph(df.att.ed, 0.15)
#' @export
assoc_graph <- function(dat, cutoff = 0) {
    J <- joint_entropy(dat)
    adj <- J$matrix
    diag(adj) <- 0
    plot_title <- paste("J", ">", cutoff, sep = " ")
    adj[adj < cutoff] <- 0
    adj[lower.tri(adj)] <- adj[upper.tri(adj)]
    ag <-
        igraph::graph_from_adjacency_matrix(adj, mode = "undirected", weighted = TRUE)
    ag.p <- ggraph::ggraph(ag, layout = "stress") +
        ggraph::geom_edge_link0(edge_colour = "grey40", edge_width = igraph::E(ag)$weight) +
        ggraph::geom_node_point(
            shape = 21,
            size = 20,
            fill = "white",
            stroke = 1
        ) +
        ggraph::geom_node_text(ggplot2::aes(label = igraph::V(ag)$name), size = 3.5) +
        ggplot2::ggtitle(plot_title) +
        ggraph::theme_graph(base_family = "") +
        ggplot2::theme(legend.position = "none") +
        ggplot2::coord_cartesian(clip = "off")

    return(ag.p)
}

#' Law Firm
#' @docType data
#' @usage data(lawdata)
#' @description This data set comes from a network study of corporate
#' law partnership that was carried out in a Northeastern US corporate law firm,
#' referred to as SG&R, 1988-1991 in New England.
#' It includes (among others) measurements of networks among the
#' 71 attorneys (partners and associates) of this firm,
#' i.e. their strong- co-worker network, advice network, friendship network,
#' and indirect control networks.
#' Various members' attributes are also part of the data set,
#' including seniority, formal status, office in which they work, gender,
#' law school attended. The ethnography, organizational and network analyses
#' of this case are available in Lazega (2001).
#'
#' \strong{Basic advice network}:
#' "Think back over the past year, consider all the lawyers in your Firm. To whom did you go for basic professional advice? For instance, you want to make sure that you are handling a case right, making a proper decision, and you want to consult someone whose professional opinions are in general of great value to you. By advice I do not mean simply technical advice."
#'
#' \strong{Friendship network:}
#' "Would you go through this list, and check the names of those you socialize with outside work. You know their family, they know yours, for instance. I do not mean all the people you are simply on a friendly level with, or people you happen to meet at Firm functions."
#'
#' \strong{Strong coworkers network:}
#'  "Because most firms like yours are also organized very informally, it is difficult to get a clear idea of how the members really work together. Think back over the past year, consider all the lawyers in your Firm. Would you go through this list and check the names of those with whom you have worked with. (By "worked with" I mean that you have spent time together on at least one case, that you have been assigned to the same case, that they read or used your work product or that you have read or used their work product; this includes professional work done within the Firm like Bar association work, administration, etc.)"
#'
#' @format List containing the following objects as numbered
#'  \enumerate{
#'   \item{adjacency matrix for advice seeking (directed)}
#'   \item{adjacency matrix for friendship (directed)}
#'   \item{adjacency matrix for cowork (undirected)}
#'   \item{dataframe with the following attributes on each lawyer:}
#' \itemize{
#' \item{`senior`: seniority (ranked from most to least senior)}
#' \item{`status`:   1=partner; 2=associate}
#' \item{`gender`: 1=man; 2=woman}
#' \item{`office`: 1=Boston; 2=Hartford; 3=Providence}
#' \item{`years`: years with the firm}
#' \item{`age`: age of attorney}
#' \item{`practice`: 1=litigation; 2=corporate}
#' \item{`lawschool`: 1=harvard/yale; 2=ucon; 3= other}
#'
#'}
#'Note: the first 36 out of 71 respondents are the partners in the firm.
#'}
#'
#'
#' @source https://www.stats.ox.ac.uk/~snijders/siena/Lazega_lawyers_data.htm
#'
#' @examples
#' data(lawdata)
#' ## assign the correct names to the objects in the list
#' adj.advice <- lawdata[[1]]
#' adj.friend <- lawdata[[2]]
#' adj.cowork <-lawdata[[3]]
#' df.att <- lawdata[[4]]
#'
#' @references
#' Emmanuel Lazega, \emph{The Collegial Phenomenon: The Social Mechanisms of Cooperation Among Peers in a Corporate Law Partnership}, Oxford University Press (2001).
#'
#' Tom A.B. Snijders, Philippa E. Pattison, Garry L. Robins, and Mark S. Handcock. New specifications for exponential random graph models. \emph{Sociological Methodology} (2006), 99-153.
"lawdata"

#' @title Prediction Power Heatmap
#' @description Creates a heatmap for visualizing prediction power from a
#' prediction power matrix.
#' @param mat matrix returned by \code{\link{prediction_power}}. Entries should
#' contain expected conditional entropies \emph{EH(Z|X,Y)}.
#' @param title character string giving the plot title.
#' @param low color for low expected conditional entropy values. Default is
#' \code{"azure4"}.
#' @param high color for high expected conditional entropy values. Default is
#' \code{"white"}.
#' @param text_size numeric value controlling the size of the cell labels.
#' Default is 2.5.
#' @return A \code{ggplot} object showing a heatmap of expected conditional
#' entropy values. Darker cells indicate lower prediction uncertainty and
#' therefore higher prediction power.
#' @details The plot visualizes expected conditional entropies
#' \deqn{EH(Z|X,Y)}
#' where \emph{Z} is the target variable and \emph{X} and \emph{Y} are predictors.
#' Diagonal entries correspond to prediction using a single predictor,
#' \emph{EH(Z|X)}, while off-diagonal entries correspond to prediction using
#' pairs of predictors, \emph{EH(Z|X,Y)}. Lower values indicate stronger
#' predictive power.
#' @seealso \code{\link{prediction_power}}, \code{\link{entropy_trivar}}
#' @examples
#' # use internal data set
#' data(lawdata)
#'
#' # extract node attributes
#' df_att <- lawdata[[4]]
#'
#' # data editing:
#' # 1. discretize 'years' and 'age' into 3 categories
#' # 2. ensure values start at 0 where needed
#' att_var <- data.frame(
#'   status    = df_att$status - 1,
#'   gender    = df_att$gender,
#'   office    = df_att$office - 1,
#'   years     = ifelse(df_att$years <= 3, 0,
#'                 ifelse(df_att$years <= 13, 1, 2)),
#'   age       = ifelse(df_att$age <= 35, 0,
#'                 ifelse(df_att$age <= 45, 1, 2)),
#'   practice  = df_att$practice,
#'   lawschool = df_att$lawschool - 1
#' )
#'
#' # compute prediction power matrix for 'status'
#' pred_mat <- prediction_power("status", att_var)
#'
#' # visualize prediction power
#' make_pred_plot(pred_mat, "Prediction Power for Status")
#' @importFrom ggplot2 .data
#' @export

make_pred_plot <- function(mat,
                           title,
                           low = "azure4",
                           high = "white",
                           text_size = 2.5) {
  df <- as.data.frame(as.table(as.matrix(mat)))
  names(df) <- c("X", "Y", "EH")

  df <- df[!is.na(df$EH), ]

  ggplot2::ggplot(df, ggplot2::aes(x = .data$Y, y = .data$X, fill = .data$EH)) +
    ggplot2::geom_tile(color = "white") +
    ggplot2::geom_text(
      ggplot2::aes(label = round(.data$EH, 2)),
      size = text_size
    ) +
    ggplot2::scale_fill_gradient(
      low = low,
      high = high,
      name = "EH"
    ) +
    ggplot2::labs(
      title = title,
      x = NULL,
      y = NULL
    ) +
    ggplot2::theme_minimal() +
    ggplot2::theme(
      axis.text.x = ggplot2::element_text(angle = 45, hjust = 1, size = 7),
      axis.text.y = ggplot2::element_text(size = 7),
      plot.title = ggplot2::element_text(size = 10)
    )
}

#' @title Trivariate Entropy
#' @description Computes trivariate entropies of all triples of discrete
#' variables in a multivariate data set.
#' @param dat dataframe with rows as observations and columns as variables.
#' Variables must all be observed or transformed categorical with finite range spaces.
#' @return Dataframe with the first three columns representing possible triples
#' of variables (\code{X}, \code{Y}, \code{Z}) and the fourth column giving
#' trivariate entropies \code{H(X,Y,Z)}.
#' @details Trivariate entropies can be used to check for functional relationships
#' and stochastic independence between triples of variables.
#'
#' The trivariate entropy \emph{H(X,Y,Z)} of three discrete random variables
#' \emph{X}, \emph{Y}, and \emph{Z} is bounded according to
#' \deqn{H(X,Y) <= H(X,Y,Z) <= H(X,Z) + H(Y,Z) - H(Z).}
#'
#' The increment between the trivariate entropy and its lower bound is equal to
#' the expected conditional entropy.
#' @author Termeh Shafie
#' @seealso \code{\link{entropy_bivar}}, \code{\link{prediction_power}}
#' @references Frank, O., & Shafie, T. (2016). Multivariate entropy analysis of network data.
#' \emph{Bulletin of Sociological Methodology/Bulletin de Méthodologie Sociologique}, 129(1), 45-63.
#' @examples
#' # use internal data set
#' data(lawdata)
#' df_att <- lawdata[[4]]
#'
#' # data editing:
#' # 1. categorize variables 'years' and 'age' into approximately
#' # equally sized groups
#' # 2. recode selected variables so categories start at 0
#' att_var <- data.frame(
#'   status    = df_att$status - 1,
#'   gender    = df_att$gender,
#'   office    = df_att$office - 1,
#'   years     = ifelse(df_att$years <= 3, 0,
#'                 ifelse(df_att$years <= 13, 1, 2)),
#'   age       = ifelse(df_att$age <= 35, 0,
#'                 ifelse(df_att$age <= 45, 1, 2)),
#'   practice  = df_att$practice,
#'   lawschool = df_att$lawschool - 1
#' )
#'
#' # calculate trivariate entropies
#' h_trivar <- entropy_trivar(att_var)
#' @export

entropy_trivar <- function(dat) {
  varname_orig <- colnames(dat)
  varname_new <- sprintf("V%d", seq_len(ncol(dat)))
  names(dat) <- varname_new

  p <- ncol(dat)
  n <- nrow(dat)
  log2_n <- log2(n)
  n_out <- choose(p, 3)

  X <- character(n_out)
  Y <- character(n_out)
  Z <- character(n_out)
  H <- numeric(n_out)

  k <- 0
  for (x in seq_len(p - 2)) {
    for (y in (x + 1):(p - 1)) {
      for (z in (y + 1):p) {
        k <- k + 1

        frq <- as.vector(table(dat[, x], dat[, y], dat[, z]))
        pos <- frq[frq > 0]
        h_tmp <- log2_n - sum(pos * log2(pos)) / n

        X[k] <- varname_orig[x]
        Y[k] <- varname_orig[y]
        Z[k] <- varname_orig[z]
        H[k] <- round(h_tmp, 3)
      }
    }
  }

  data.frame(
    X = X,
    Y = Y,
    Z = Z,
    `H(X,Y,Z)` = H,
    check.names = FALSE,
    stringsAsFactors = FALSE
  )
}

test_that("trivariate entropy works", {
  data(lawdata)
  df.att <- lawdata[[4]]

  df.att.ed <- data.frame(
    status   = df.att$status,
    gender   = df.att$gender,
    office   = df.att$office-1,
    years    = ifelse(df.att$years<=3,0,
                      ifelse(df.att$years<=13,1,2)),
    age      = ifelse(df.att$age<=35,0,
                      ifelse(df.att$age<=45,1,2)),
    practice = df.att$practice,
    lawschool= df.att$lawschool-1)

  H.tri <- entropy_trivar(df.att.ed)
  result <- c(2.745, 2.63, 2.897, 2.67, 3.123, 2.995, 3.194, 3.032, 3.448, 3.079)
  expect_identical(H.tri[["H(X,Y,Z)"]][1:10], result)
})

---
title: "Prediction Power Heatmaps"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Prediction Power Heatmaps}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(out.width = "100%", cache = FALSE)
```

The function `make_pred_plot()` visualizes the output from `prediction_power()` as a heatmap. Each cell shows an expected conditional entropy value, where lower values indicate stronger prediction power. Diagonal entries correspond to prediction using a single predictor, while off-diagonal entries correspond to prediction using pairs of predictors.

```{r load-lib}
library(netropy)
```

We first edit the node attributes so that all variables have finite categorical range spaces. The variables years and age are discretized into three categories.

```{r data-edit}
df_att <- lawdata[[4]]
att_var <- data.frame(
  status    = df_att$status - 1,
  gender    = df_att$gender,
  office    = df_att$office - 1,
  years     = ifelse(df_att$years <= 3, 0,
                ifelse(df_att$years <= 13, 1, 2)),
  age       = ifelse(df_att$age <= 35, 0,
                ifelse(df_att$age <= 45, 1, 2)),
  practice  = df_att$practice,
  lawschool = df_att$lawschool - 1
)
```
The first rows of the edited attribute data are:

```{r}
head(att_var)
```

## Prediction Power
Assume we are interested in predicting `status`, which indicates whether a lawyer is an associate or a partner. We first compute the prediction power matrix:

```{r pred-pow}
pred_status <- prediction_power("status", att_var)
pred_status
```

### Heatmap Visualization
The matrix can be visualized using `make_pred_plot()`:

```{r pred-plot, fig.height=7, fig.width=8}
make_pred_plot(pred_status, "Prediction Power for Status")
```

Darker cells indicate lower expected conditional entropy and therefore stronger prediction power. The diagonal entries show prediction based on one variable, while the off-diagonal entries show prediction based on pairs of variables.

### Changing Plot Colors
The colors can be adjusted using the `low` and `high` arguments. For example:
```{r col-change, fig.height=7, fig.width=8}
make_pred_plot(
  pred_status,
  "Prediction Power for Status",
  low = "steelblue",
  high = "white"
)
```

### Changing Text Size
The size of the cell labels can be controlled with `text_size`:
```{r text-change, fig.height=7, fig.width=8}
make_pred_plot(
  pred_status,
  "Prediction Power for Status",
  text_size = 6
)
```


## References
> Frank, O., & Shafie, T. (2016). Multivariate entropy analysis of network data.
*Bulletin of Sociological Methodology/Bulletin de Méthodologie Sociologique*, 129(1), 45-63. [link](https://doi.org/10.1177%2F0759106315615511)

---
title: "Prediction Power Based on Expected Conditional Entropies"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Prediction Power Based on Expected Conditional Entropies}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(out.width = "100%",
  cache = FALSE
)
```
In the section on [univariate, bivariate and trivariate entropies](univariate_bivariate_trivariate.html),
we saw that the bivariate entropy of two variables $X$ and $Y$ is bounded according to 
$$H(X) \leq H(X,Y) \leq H(X)+H(Y) \ .$$
The increment between the lower bound and the bivariate entropy is equal to the expected conditional entropy
$$EH(Y|X)=H(X,Y)-H(X)$$
which is a measure of how far from functional dependence $X\rightarrow Y$ (which means that that $X$ uniquely determines $Y$) we are. This measure is equal to 0 if and only if $p(x,y) = p(x,+)$ meaning $X$ uniquely determines $Y$.

Similarly, trivariate entropies for triples of variables $X,Y,Z$ are bounded by
$$ H(X,Y) \leq H(X,Y,Z) \leq H(X,Z) + H(Y,Z) - H(Z) $$
and the increment between the trivariate entropy and its lower bound is equal to the expected conditional entropy given by 
$$EH(Z|X,Y) = H(X,Y,Z)-H(X,Y)$$
which is non-negative and equal to 0 if and only if there is functional dependence $(X,Y)\rightarrow Z$. Thus, $EH(Z|X,Y)$ measures the prediction uncertainty when $(X,Y)$ is used to predict $Z$. 

$EH=EH(Z|X,Y)$ is a logarithmic measure of how many outcomes there are of $Z$ on average when the outcomes are given for $X$ and $Y$ . If $EH$ is rounded to its closest integer, we get an unambiguous prediction value for $Z$ based on predictors $X$ and $Y$ when $EH < 0.5$ and two prediction values for $Z$ when $0.5\leq EH < 1.5$ etc. 
Thus, prediction power is a decreasing function of $EH$. 

________________________________________________________________________________

## Example: prediction power based on expected conditional entropies
```{r load_library, eval=TRUE}
library(netropy)
```

We create a dataframe `dyad.var` consisting of dyad variables as described and created in [variable domains and data editing](variable_domains.html). Similar analyses can be performed on observed and/or transformed dataframes with vertex or triad variables.

```{r load_data, eval=TRUE, include=FALSE}
data(lawdata) 
adj.advice <- lawdata[[1]]
adj.friend <- lawdata[[2]]
adj.cowork <-lawdata[[3]]
df.att <- lawdata[[4]]
```
```{r edit_data, eval=TRUE, include=FALSE, results ='hide'}
att.var <-
  data.frame(
    status   = df.att$status-1,
    gender   = df.att$gender,
    office   = df.att$office-1,
    years    = ifelse(df.att$years<=3,0,
                      ifelse(df.att$years<=13,1,2)),
    age      = ifelse(df.att$age<=35,0,
                      ifelse(df.att$age<=45,1,2)),
    practice = df.att$practice,
    lawschool= df.att$lawschool-1
    )
dyad.status    <- get_dyad_var(att.var$status, type = 'att')
dyad.gender    <- get_dyad_var(att.var$gender, type = 'att')
dyad.office    <- get_dyad_var(att.var$office, type = 'att')
dyad.years     <- get_dyad_var(att.var$years, type = 'att')
dyad.age       <- get_dyad_var(att.var$age, type = 'att')
dyad.practice  <- get_dyad_var(att.var$practice, type = 'att')
dyad.lawschool <- get_dyad_var(att.var$lawschool, type = 'att')
dyad.cwk    <- get_dyad_var(adj.cowork, type = 'tie')
dyad.adv    <- get_dyad_var(adj.advice, type = 'tie')
dyad.frn    <- get_dyad_var(adj.friend, type = 'tie')
dyad.var <-
  data.frame(cbind(status    = dyad.status$var,
                  gender    = dyad.gender$var,
                  office    = dyad.office$var,
                  years     = dyad.years$var,
                  age       = dyad.age$var,
                  practice  = dyad.practice$var,
                  lawschool = dyad.lawschool$var,
                  cowork    = dyad.cwk$var,
                  advice    = dyad.adv$var,
                  friend    = dyad.frn$var)
                  )
```
```{r show_data, eval=TRUE, include=TRUE, results ='markup'}
head(dyad.var)
```
The function `prediction_power()` computes prediction power when pairs of variables in a given dataframe are used to predict a third variable from the same dataframe. The variable to be predicted and the dataframe in which this variable also is part of is given as input arguments, and the output is an upper triangular matrix giving the expected conditional entropies of pairs of row and column variables of the matrix, i.e. $EH(Z|X,Y)$. The diagonal gives $EH(Z|X)$ , that is when only one variable as a predictor. Note that `NA`'s are in the row and column representing the variable being predicted.

Assume we are interested in predicting variable `status` (that is whether a lawyer in the data set is an associate or partner). This is done by running the following:
```{r predpow1, eval=TRUE, include=TRUE}
prediction_power('status', dyad.var)
```
For better readability, the powers of different predictors can be conveniently compared by using prediction plots that display a color matrix with rows for $X$ and columns for $Y$ with darker colors in the cells when we have higher prediction power for $Z$. These can be created with the function `make_pred_plot()`.

## References
> Frank, O., & Shafie, T. (2016). Multivariate entropy analysis of network data.
*Bulletin of Sociological Methodology/Bulletin de Méthodologie Sociologique*, 129(1), 45-63. [link](https://doi.org/10.1177%2F0759106315615511)

---
title: "Divergence Tests of Goodness of Fit"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Divergence Tests of Goodness of Fit}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(out.width = "100%", cache = FALSE)
```


Association graphs can be used to suggest structural models of multivariate dependence. The function `div_gof()` provides divergence-based goodness-of-fit tests for several such hypotheses, including uniformity, pairwise independence, conditional independence, and nested model comparisons.

```{r load_library, eval=TRUE}
library(netropy)
```

## Example: 
We use the law firm network data included in the package.

```{r load_data, eval=TRUE, include=FALSE}
data(lawdata)
adj_advice <- lawdata[[1]]
adj_friend <- lawdata[[2]]
adj_cowork <- lawdata[[3]]
df_att <- lawdata[[4]]
```
We first edit the node attributes and transform them into dyad variables.
```{r edit_data, eval=TRUE, include=FALSE, results ='hide'}
att_var <- data.frame(
  status    = df_att$status - 1,
  gender    = df_att$gender,
  office    = df_att$office - 1,
  years     = ifelse(df_att$years <= 3, 0,
                ifelse(df_att$years <= 13, 1, 2)),
  age       = ifelse(df_att$age <= 35, 0,
                ifelse(df_att$age <= 45, 1, 2)),
  practice  = df_att$practice,
  lawschool = df_att$lawschool - 1
)

dyad_status    <- get_dyad_var(att_var$status, type = "att")
dyad_gender    <- get_dyad_var(att_var$gender, type = "att")
dyad_office    <- get_dyad_var(att_var$office, type = "att")
dyad_years     <- get_dyad_var(att_var$years, type = "att")
dyad_age       <- get_dyad_var(att_var$age, type = "att")
dyad_practice  <- get_dyad_var(att_var$practice, type = "att")
dyad_lawschool <- get_dyad_var(att_var$lawschool, type = "att")

dyad_cwk <- get_dyad_var(adj_cowork, type = "tie")
dyad_adv <- get_dyad_var(adj_advice, type = "tie")
dyad_frn <- get_dyad_var(adj_friend, type = "tie")

dyad_var <- data.frame(
  status    = dyad_status$var,
  gender    = dyad_gender$var,
  office    = dyad_office$var,
  years     = dyad_years$var,
  age       = dyad_age$var,
  practice  = dyad_practice$var,
  lawschool = dyad_lawschool$var,
  cowork    = dyad_cwk$var,
  advice    = dyad_adv$var,
  friend    = dyad_frn$var
)
```

The first rows of the dyad-level data are:
```{r show_data, eval=TRUE, include=TRUE}
head(dyad_var)
```

### Conditional Independence
A model of substantive interest is whether friendship and co-working are conditionally independent given advice:

$$
\texttt{friend} \perp \texttt{cowork} \mid \texttt{advice}.
$$

This can be tested by specifying `var_cond`:
```{r cond_ind}
div_gof(
  dat = dyad_var,
  var1 = "friend",
  var2 = "cowork",
  var_cond = "advice"
)
```

### Pairwise Indpendence
If `var_cond` is omitted, `div_gof()` tests ordinary pairwise independence:

$$
\texttt{friend} \perp \texttt{cowork}.
$$
```{r pair_ind}
div_gof(
  dat = dyad_var,
  var1 = "friend",
  var2 = "cowork"
)
```

### Uniformity

The function can also test whether a single variable is uniformly distributed across its observed categories. This is specified using `var_uniform`:
```{r unif}
div_gof(
  dat = dyad_var,
  var_uniform = "friend"
)
```

### Nested Model Comparison
Reduced models can also be compared to the saturated empirical model. The saturated model is represented by divergence `D = 0` and degrees of freedom `df = 0`.

```{r nest_ind1}
m_full <- list(D = 0, df = 0)

m_reduced <- div_gof(
  dat = dyad_var,
  var1 = "friend",
  var2 = "cowork"
)

div_gof(
  dat = dyad_var,
  model_full = m_full,
  model_reduced = list(D = m_reduced$D, df = m_reduced$df)
)
```
Similarly, we can compare a conditional independence model to the saturated empirical model:

```{r nest_ind2}
m_reduced <- div_gof(
  dat = dyad_var,
  var1 = "friend",
  var2 = "cowork",
  var_cond = "advice"
)

div_gof(
  dat = dyad_var,
  model_full = m_full,
  model_reduced = list(D = m_reduced$D, df = m_reduced$df)
)
```


## References
> Frank, O., & Shafie, T. (2016). Multivariate entropy analysis of network data.
*Bulletin of Sociological Methodology/Bulletin de Méthodologie Sociologique*, 129(1), 45-63. [link](https://doi.org/10.1177%2F0759106315615511)

---
title: "Univariate, Bivariate, Trivariate, and Tetravariate Entropies"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Univariate, Bivariate and Trivariate Entropies}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(out.width = "100%",
  cache = FALSE
)
```
The univariate entropy for discrete variable $X$ with $r$ outcomes is defined by
$$H(X) = \sum_x p(x) \log_2\frac{1}{p(x)} $$
with which we can check for redundancy and uniformity: a discrete random variable with minimal zero entropy has no uncertainty and is always equal to the same single outcome. Thus, it is a constant that contributes nothing to further analysis and can be omitted. Maximum entropy is $\log_2r$ and it corresponds to a uniform probability distribution over the outcomes.

The bivariate entropy for discrete variable $X$ and $Y$ is defined by
$$H(X,Y) = \sum_x \sum_y p(x,y) \log_2\frac{1}{p(x,y)}$$
with which we can check for redundancy, functional relationships and stochastic independence between pairs of variables. It is bounded according to 
$$H(X) \leq H(X,Y) \leq H(X)+H(Y)$$
where we have 

- equality to the left iff there is a functional relationship $Y = f(X)$ such that each unique outcome of $X$ yields a unique outcome of $Y$

- equality to the right iff $X$ and $Y$ are stochastically independent  $X\bot Y$ such that the probability of any bivariate outcome is the product of the probabilities of the univariate outcomes.

Note that when the bivariate entropy of two variables is equal to the univariate entropy of either one alone, then one of these variables should be omitted as they are redundant providing no additional information.

These results on bivariate entropies are directly linked to [joint entropies and association graphs](joint_entropies.html). 

Similarly, trivariate entropies (and higher order entropies) allows us to check for functional relationships and stochastic independence between three (or more) variables. The trivariate entropy of three variables $X$, $Y$ and $Z$ is defined by
$$H(X,Y,Z) = \sum_x \sum_y \sum_z p(x,y,z) \log_2\frac{1}{p(x,y,z)}$$

and bounded by
$$ H(X,Y) \leq H(X,Y,Z) \leq H(X,Z) + H(Y,Z) - H(Z). $$

The results on bivariate and trivariate entropies are directly linked to [prediction power and expected conditional entropies](prediction_power.html).


Tetravariate entropies extend the same idea to quadruples of variables. For four variables $X$, $Y$, $Z$, and $U$, the tetravariate entropy is denoted by

$$
H(X,Y,Z,U).
$$

The function `entropy_tetravar()` computes tetravariate entropy quantities for all quadruples of variables in a dataframe. In addition to $H(X,Y,Z,U)$, it returns expected conditional entropies and expected conditional joint entropies, such as

$$
EH(U \mid X,Y,Z) = H(X,Y,Z,U) - H(X,Y,Z),
$$

and

$$
EJ(X,Y \mid Z,U) = H(X,Z,U) + H(Y,Z,U) - H(Z,U) - H(X,Y,Z,U).
$$

These quantities are useful for studying prediction uncertainty and conditional dependence among four variables.


Examples of computing univariate, bivariate, trivariate and tetravariate entropies are given in the following. 

________________________________________________________________________________

## Example: univariate and bivariate entropies
```{r load_library, eval=TRUE}
library(netropy)
```

We create a dataframe `dyad.var` consisting of dyad variables as described and created in [variable domains and data editing](variable_domains.html). Similar analyses can be perfomed on observed and/or transformed dataframes with vertex or triad variables.

```{r load_data, eval=TRUE, include=FALSE}
data(lawdata) 
adj.advice <- lawdata[[1]]
adj.friend <- lawdata[[2]]
adj.cowork <-lawdata[[3]]
df.att <- lawdata[[4]]
```
```{r edit_data, eval=TRUE, include=FALSE, results ='hide'}
att.var <-
  data.frame(
    status   = df.att$status-1,
    gender   = df.att$gender,
    office   = df.att$office-1,
    years    = ifelse(df.att$years<=3,0,
                      ifelse(df.att$years<=13,1,2)),
    age      = ifelse(df.att$age<=35,0,
                      ifelse(df.att$age<=45,1,2)),
    practice = df.att$practice,
    lawschool= df.att$lawschool-1
    )
dyad.status    <- get_dyad_var(att.var$status, type = 'att')
dyad.gender    <- get_dyad_var(att.var$gender, type = 'att')
dyad.office    <- get_dyad_var(att.var$office, type = 'att')
dyad.years     <- get_dyad_var(att.var$years, type = 'att')
dyad.age       <- get_dyad_var(att.var$age, type = 'att')
dyad.practice  <- get_dyad_var(att.var$practice, type = 'att')
dyad.lawschool <- get_dyad_var(att.var$lawschool, type = 'att')
dyad.cwk    <- get_dyad_var(adj.cowork, type = 'tie')
dyad.adv    <- get_dyad_var(adj.advice, type = 'tie')
dyad.frn    <- get_dyad_var(adj.friend, type = 'tie')
dyad.var <-
  data.frame(cbind(status    = dyad.status$var,
                  gender    = dyad.gender$var,
                  office    = dyad.office$var,
                  years     = dyad.years$var,
                  age       = dyad.age$var,
                  practice  = dyad.practice$var,
                  lawschool = dyad.lawschool$var,
                  cowork    = dyad.cwk$var,
                  advice    = dyad.adv$var,
                  friend    = dyad.frn$var)
                  )
```
```{r show_data, eval=TRUE, include=TRUE, results ='markup'}
head(dyad.var)
```

The function `entropy_bivar()` computes the bivariate entropies of all pairs of variables in the dataframe. The output is given as an upper triangular matrix with cells giving the bivariate entropies of row and column variables. The diagonal thus gives the univariate entropies for each variable in the dataframe:
```{r biv_ent, eval=TRUE, include=TRUE, results ='markup'}
entropy_bivar(dyad.var)
```
## Example: redundant variables
Bivariate entropies can be used to detect redundant variables that should be omitted from the dataframe for further analysis. When calculating bivariate entropies, one can check of whether the diagonal values are equal to any of the other values in the rows an columns. As seen above, the dataframe `dyad.var`  has no redundant variables. This can also be checked using the function `redundancy()` which yields  a binary matrix as output indicating which row and column variables are hold the same information:
```{r red, eval=TRUE, include=TRUE, results ='markup'}
redundancy(dyad.var)
```
To illustrate an example with redundancy, we use the dataframe `att.var` with node attributes as described and created in [variable domains and data editing](variable_domains.html). Note however that we now keep the variable `senior` in this dataframe:

```{r edit_data2, eval=TRUE, include=FALSE, results ='hide'}
att.var <-
  data.frame(
    senior   = df.att$senior,
    status   = df.att$status-1,
    gender   = df.att$gender,
    office   = df.att$office-1,
    years    = ifelse(df.att$years<=3,0,
                      ifelse(df.att$years<=13,1,2)),
    age      = ifelse(df.att$age<=35,0,
                      ifelse(df.att$age<=45,1,2)),
    practice = df.att$practice,
    lawschool= df.att$lawschool-1
    )
```

```{r show_data2, eval=TRUE, include=TRUE, results ='markup'}
head(att.var)
```
Checking redundancy on this dataframe yields the following output:
```{r red2, eval=TRUE, include=TRUE, results ='markup'}
redundancy(att.var)
```
As seen,  `senior` has been flagged as a redundant variable which is not surprising since it only consists of unique values. This redudancy can also be noted by computing the bivariate entropies and noting that the univariate entropy for this variable is equal to the bivariate entropies of pairs including this variable:
```{r biv_ent2, eval=TRUE, include=TRUE, results ='markup'}
entropy_bivar(att.var)
```
## Example: trivariate entropies
Trivariate entropies can be computed using the function `entropy_trivar()` which returns a dataframe with the first three columns `X`, `Y`, and `Z` giving the variable names for each triple from the dataframe in question, and their entropies `H(X,Y,Z)` as the fourth column. We illustrated this on the dataframe `dyad.var`:
```{r triv_ent, eval=TRUE, include=TRUE, results ='markup'}
entropy_trivar(dyad.var)
```

## Example: tetravariate entropies
To compute tetravariate entropy quantities for five selected dyad variables, we use:

```{r tetrav_ent, eval=TRUE, include=TRUE, results='markup'}
entropy_tetravar(
  dat = dyad.var[, c("gender", "years", "age", "office", "practice")]
)
```

## References
> Frank, O., & Shafie, T. (2016). Multivariate entropy analysis of network data.
*Bulletin of Sociological Methodology/Bulletin de Méthodologie Sociologique*, 129(1), 45-63. [link](https://doi.org/10.1177%2F0759106315615511)

---
title: "Joint Entropies and Association Graphs"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Joint Entropies and Association Graphs}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(out.width = "100%",
  cache = FALSE
)
```
In the section on [univariate, bivariate and trivariate entropies](univariate_bivariate_trivariate.html),
we saw that the bivariate entropy of two variables $X$ and $Y$ is bounded according to 
$$H(X) \leq H(X,Y) \leq H(X)+H(Y) \ .$$
The increment between the upper bound and the bivariate entropy is equal to the joint entropy given by
$$J(X,Y) = H(X)+H(Y)-H(X,Y)$$
and is a non-negative measure of dependence or association between variable $X$ and $Y$. It is equal to 0 if and only if $X$ and $Y$ are stochastically independent  $X\bot Y$ such that the probability of any bivariate outcome is the product of the probabilities of the univariate outcomes, i.e. $p(x,y)=p(x,+)p(+,y)$.

Trivariate entropies for triples of variables $X,Y,Z$ are bounded by
$$ H(X,Y) \leq H(X,Y,Z) \leq H(X,Z) + H(Y,Z) - H(Z) $$
The increment between the upper bound and the trivariate entropy is equal to the expected joint entropy given by
$$EJ(X,Y|Z) = H(X,Z)+H(Y,Z)-H(Z)-H(X,Y,Z)$$
which is non-negative and equal to 0 if and only if we have conditional independence of the form $X\bot Y | Z$. Thus, it can be used to measure the deviation from conditional independence.

The joint entropies $J(X,Y)$ for all pairs of variables $X$ and $Y$ are used to check dependence structure by looking at a sequence of association graphs constructed with nodes representing the variables and links between any pair of nodes corresponding to two variables with joint entropy above a chosen threshold. By successively lowering the threshold from the maximum joint entropy to smaller occurring values, the sequence of graphs get more and more links. Connected components that are cliques represent dependent subsets of variables, and different components represent independent subsets of variables. Conditional independence between subsets of variables can be identified by omitting the subset corresponding to the conditioning variables. A threshold that gives a graph with reasonably many small independent or conditionally independent subsets of variables can be considered to represent a multivariate model for further testing.

________________________________________________________________________________

## Example: joint entropies and association graphs
```{r load_library, eval=TRUE}
library(netropy)
```

We create a dataframe `dyad.var` consisting of dyad variables as described and created in [variable domains and data editing](variable_domains.html). Similar analyses can be performed on observed and/or transformed dataframes with vertex or triad variables.

```{r load_data, eval=TRUE, include=FALSE}
data(lawdata) 
adj.advice <- lawdata[[1]]
adj.friend <- lawdata[[2]]
adj.cowork <-lawdata[[3]]
df.att <- lawdata[[4]]
```
```{r edit_data, eval=TRUE, include=FALSE, results ='hide'}
att.var <-
  data.frame(
    status   = df.att$status-1,
    gender   = df.att$gender,
    office   = df.att$office-1,
    years    = ifelse(df.att$years<=3,0,
                      ifelse(df.att$years<=13,1,2)),
    age      = ifelse(df.att$age<=35,0,
                      ifelse(df.att$age<=45,1,2)),
    practice = df.att$practice,
    lawschool= df.att$lawschool-1
    )
dyad.status    <- get_dyad_var(att.var$status, type = 'att')
dyad.gender    <- get_dyad_var(att.var$gender, type = 'att')
dyad.office    <- get_dyad_var(att.var$office, type = 'att')
dyad.years     <- get_dyad_var(att.var$years, type = 'att')
dyad.age       <- get_dyad_var(att.var$age, type = 'att')
dyad.practice  <- get_dyad_var(att.var$practice, type = 'att')
dyad.lawschool <- get_dyad_var(att.var$lawschool, type = 'att')
dyad.cwk    <- get_dyad_var(adj.cowork, type = 'tie')
dyad.adv    <- get_dyad_var(adj.advice, type = 'tie')
dyad.frn    <- get_dyad_var(adj.friend, type = 'tie')
dyad.var <-
  data.frame(cbind(status    = dyad.status$var,
                  gender    = dyad.gender$var,
                  office    = dyad.office$var,
                  years     = dyad.years$var,
                  age       = dyad.age$var,
                  practice  = dyad.practice$var,
                  lawschool = dyad.lawschool$var,
                  cowork    = dyad.cwk$var,
                  advice    = dyad.adv$var,
                  friend    = dyad.frn$var)
                  )
```
```{r show_data, eval=TRUE, include=TRUE, results ='markup'}
head(dyad.var)
```

The function `joint_entropy()` computes the joint entropies between all pairs of variables in a given dataframe and returns a list consisting of the upper triangular joint entropy matrix (univariate entropies in the diagonal) and a dataframe giving the frequency distributions of unique joint entropy values. A function argument specifies the precision given in number of decimals for which the frequency distribution of unique entropy values is created (default is 3). Applying the function on the  dataframe `dyad.var` with two decimals:
```{r joint_ent, eval=TRUE, include=TRUE, results ='markup'}
J <- joint_entropy(dyad.var, 2)
J$matrix
J$freq
```
As seen, the strongest association is between the variables `status` and `years` with joint entropy values of 0.79. We have independence (joint entropy value of 0) between two pairs of variables: (`status`,`practice`), (`practise`,`gender`), (`cowork`,`gender`),and  (`cowork`,`lawschool`).

These results can be illustrated in a association graph using the function `assoc_graph()` which returns a `ggraph` object in which nodes represent variables and links represent strength of association (thicker links indicate stronger dependence). To use the function we need to load the `ggraph` library and to determine a threshold which the graph drawn is based on. We set it to 0.15 so that we only visualize the strongest associations:
```{r assoc_g, eval=TRUE, fig.align='center',fig.width=9, message =FALSE}
library(ggraph)
assoc_graph(dyad.var, 0.15)
```

Given this threshold, we see isolated and disconnected nodes representing independent variables. 
We note strong dependence between the three dyadic variables `status`,`years` and `age`, but also a somewhat strong dependence among the three variables  `lawschool`, `years` and `age`, and the three variables `status`, `years` and `gender`. The association graph can also be interpreted as a tendency for  relations `cowork` and `friend` to be independent conditionally on relation `advice`, that is, any dependence between dyad variables `cowork` and `friend` is explained by `advice`.


## References
> Frank, O., & Shafie, T. (2016). Multivariate entropy analysis of network data.
*Bulletin of Sociological Methodology/Bulletin de Méthodologie Sociologique*, 129(1), 45-63. [link](https://doi.org/10.1177%2F0759106315615511)

---
title: "Variable Domains and Data Editing"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Variable Domains and Data Editing}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(out.width = "100%",
  cache = FALSE
)
```

Data editing should guarantee that all variables are carefully defined with specified domains where they are observed or measured and with specified range spaces where their possible values are given. A usual set-up is that all variables have the same domain, for instance a set of individuals for which several attributes are registered. If a binary relationship like friendship or not friendship is registered for all pairs of individuals, then the domain for this relationship is the set of dyads (pairs without regard to order) of individuals.

Variables with different domains can sometimes be combined. For instance, node or vertex variable $X$ with value $X_u$ for node $u$ can be extended to the domain of dyads by defining it as the pair $X_{uv} = (X_u,X_v)$ for dyad $(u,v)$. Examples of such variable transformations are given in Frank & Shafie (2016) ([link](https://doi.org/10.1177%2F0759106315615511)). 

Note that is also possible to create a new variable on units from a tie variable $Y$ with values $Y_{uv}$ on dyads by aggregating in some way the values on dyads incident to a node $u$ to get a value $Y_u$ at that node. For instance, the edge indicators in a graph can be aggregated to degrees or other centrality measures at the vertices.

Thus, variables can be observed and transformed on three variable domains: vertex, dyad and triad variables. This is exemplified in the following.

________________________________________________________________________________

## Load internal data set

```{r eval=TRUE}
library('netropy')
```

To load the internal data set ([link](https://www.stats.ox.ac.uk/~snijders/siena/Lazega_lawyers_data.htm)), extract each object and assign the correct names to them:
```{r load_data, eval=TRUE}
data(lawdata) 
adj.advice <- lawdata[[1]]
adj.friend <- lawdata[[2]]
adj.cowork <-lawdata[[3]]
df.att <- lawdata[[4]]
```

## Example: observed and transformed vertex variables
Observed node variables are the attributes in the data frame `df.att` with 71 observations. All variables, except `years` and `age`, are categorical with finite range spaces and therefore kept in their original form. The variable `years` and `age` need to be categorized using their cumulative distribution functions (cdf) and creating approximately equally sized categories. 
These cdf's can be obtained by
```{r cdf, eval=TRUE, include=TRUE, results ='markup'}
# for years:
x <- table(df.att$years) 
values<-as.numeric(names(x))
prop.x <- round(prop.table(x),2)
cum.prop=cumsum(prop.x)
frq.years = data.frame(value=values,freq=as.vector(x),
                       rel.freq=as.vector(prop.x), 
                       cum.rel.freq=as.vector(cum.prop))

# for age:
x <- table(df.att$age) 
values<-as.numeric(names(x))
prop.x <- round(prop.table(x),2)
cum.prop=cumsum(prop.x)
frq.age = data.frame(value=values,freq=as.vector(x),
                       rel.freq=as.vector(prop.x), 
                       cum.rel.freq=as.vector(cum.prop))
```
By looking at these cdf's we can find values with which we base the categories on:
```{r cdf2, eval=TRUE, include=TRUE, results ='markup'}
frq.years
frq.age
```
We base the categorization of these two variables variables on values yielding three approximately equally sized categories (i.e. approximately 30% of the cdf) and merge all variables into a new dataframe `att_var`:
```{r edit_data1, eval=TRUE, include=TRUE, results ='markup'}
att.var <-
  data.frame(
    status   = df.att$status-1,
    gender   = df.att$gender,
    office   = df.att$office-1,
    years    = ifelse(df.att$years <= 3,0,
                      ifelse(df.att$years <= 13,1,2)),
    age      = ifelse(df.att$age <= 35,0,
                      ifelse(df.att$age <= 45,1,2)),
    practice = df.att$practice,
    lawschool= df.att$lawschool-1
    )
head(att.var)
```
Note that we for sake of consistency also edit all variables such that their outcomes start from the value 0. The variable `senior` only has unique values, thus it is redundant and omitted (later we illustrate how such redundant variables can be detected using bivariate entropies).

To transform observed dyad variables into node variables, node degrees of each network (in- and out-degree for directed advice and friendship) can be computed and categorized as shown above for `years` and `age`. 

## Example: observed and transformed dyad variables
Dyad variables are given as pairs of incident vertex variables with $\binom{71}{2}=2485$ observations (number of rows in the dataframes created in the following). Observed node attribute in the dataframe `att_var` are thus given by pairs of individual attributes. For example,  `status` with binary outcomes  is transformed into dyads having 4 possible outcomes $(0,0), (0,1), (1,0), (1,1)$ and `office` with three categorical outcomes gives dyads with 9 possible outcome $(0,0), (0,1), (0,2), (1,0), (1,1), (1,2),(2,0),(2,1),(2,2)$. 
These transformations can be done using the function  `get_dyad_variables()` for each vertex variable using the argument `type = att` which specifies that we are using vertex attributes as input variable:
```{r edit_data2, eval=TRUE, include=TRUE, results ='markup'}
dyad.status    <- get_dyad_var(att.var$status, type = 'att')
dyad.gender    <- get_dyad_var(att.var$gender, type = 'att')
dyad.office    <- get_dyad_var(att.var$office, type = 'att')
dyad.years     <- get_dyad_var(att.var$years, type = 'att')
dyad.age       <- get_dyad_var(att.var$age, type = 'att')
dyad.practice  <- get_dyad_var(att.var$practice, type = 'att')
dyad.lawschool <- get_dyad_var(att.var$lawschool, type = 'att')
```
Note that the outcomes are recoded to numerical values to avoid character objects when performing the entropy analysis (in practice though, the actual values of variables are irrelevant for the entropy analysis as we only care about frequencies of occurrence). Thus, `status` has outcomes  0-3 and  `office` has 0-8.

Similarly, dyad variables can be created based on observed ties.
For the undirected edges, we use indicator variables read directly from the adjacency matrix for the dyad in question, while for the directed ones (`advice` and `friendship`) we have pairs of indicators representing sending and receiving ties with 4 possible outcomes:
```{r edit_data3, eval=TRUE, include=TRUE, message = FALSE, results ='markup'}
dyad.cwk    <- get_dyad_var(adj.cowork, type = 'tie')
dyad.adv    <- get_dyad_var(adj.advice, type = 'tie')
dyad.frn    <- get_dyad_var(adj.friend, type = 'tie')
```
All 10 dyad variables are merged into one data frame for subsequent entropy analysis:
```{r edit_data4, eval=TRUE, include=TRUE, results ='markup'}
dyad.var <-
  data.frame(cbind(status   = dyad.status$var,
                  gender    = dyad.gender$var,
                  office    = dyad.office$var,
                  years     = dyad.years$var,
                  age       = dyad.age$var,
                  practice  = dyad.practice$var,
                  lawschool = dyad.lawschool$var,
                  cowork    = dyad.cwk$var,
                  advice    = dyad.adv$var,
                  friend    = dyad.frn$var)
                  )
head(dyad.var)
```

## Example: transformed triad variables
A similar function  `get_triad_var()` is implemented for transforming vertex variables and different relation types into triad variables.
These triad variables have $\binom{71}{3}=57155$ observations in the law data set and are given as triples of individual attributes or by the relations among the three nodes. Similarly as for dyad variables, we call the function and specify argument for type of variable (a column vector as input when considering vertex attributes and an adjacency matrix when considering ties). For the vertex variables we thus obtain the triad variables by:
```{r edit_data5, eval=TRUE, include=TRUE, results ='markup'}
triad.status    <- get_triad_var(att.var$status, type = 'att')
triad.gender    <- get_triad_var(att.var$gender, type = 'att')
triad.office    <- get_triad_var(att.var$office, type = 'att')
triad.years     <- get_triad_var(att.var$years, type = 'att')
triad.age       <- get_triad_var(att.var$age, type = 'att')
triad.practice  <- get_triad_var(att.var$practice, type = 'att')
triad.lawschool <- get_triad_var(att.var$lawschool,type = 'att')
```
Note that binary attributes have 8 possible triadic outcomes 
$$ (0, 0, 0), (1, 0, 0), (0, 1, 0), (1, 1, 0), (0, 0, 1), (1, 0, 1), (0, 1, 1), (1, 1, 1)$$
coded 0-7 and attributes with three possible outcomes will yield triads with 27 possible outcomes coded 0-26. 


The undirected ties are transformed from having binary outcomes into triad variables with 8 possible outcomes, and directed ties are transformed from having 4 possible outcomes into triad variables with 64 possible outcome representing possible triadic combinations of sending and receiving ties.
```{r edit_data6, eval=TRUE, include=TRUE, message = FALSE, results ='markup'}
triad.cwk    <- get_triad_var(adj.cowork, type = 'tie')
triad.adv    <- get_triad_var(adj.advice, type = 'tie')
triad.frn    <- get_triad_var(adj.friend, type = 'tie')
```

All triad variables are then merged into one data frame for subsequent entropy analysis.
```{r edit_data7, eval=TRUE, include=TRUE, results ='markup'}
triad.var <- data.frame(cbind(
             status    = triad.status$var,
             gender    = triad.gender$var,
             office    = triad.office$var,
             years     = triad.years$var,
             age       = triad.age$var,
             practice  = triad.practice$var,
             lawschool = triad.lawschool$var,
             cowork    = triad.cwk$var,
             advice    = triad.adv$var,
             friend    = triad.frn$var)
             )
head(triad.var)
```

## References
> Frank, O., & Shafie, T. (2016). Multivariate entropy analysis of network data.
*Bulletin of Sociological Methodology/Bulletin de Méthodologie Sociologique*, 129(1), 45-63. [link](https://doi.org/10.1177%2F0759106315615511)

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/prediction_power.R
\name{prediction_power}
\alias{prediction_power}
\title{Prediction Power}
\usage{
prediction_power(var, dat)
}
\arguments{
\item{var}{character string representing the variable in
dataframe \code{dat} to be predicted by pairs of other variables in the dataframe \code{dat}.}

\item{dat}{dataframe with rows as observations and columns as variables.
Variables must all be observed or transformed categorical with finite range spaces.}
}
\value{
Upper triangular matrix giving the expected conditional entropies of pairs of variables
given as rows and columns of the matrix. The diagonal gives \emph{EH(Z|X) = H(X,Z) - H(X)}, that is
when only one variable is used to predict \code{var}. Note that \code{NA}'s are in the entire
row and column representing the variable being predicted.
}
\description{
Computes prediction power when pairs of variables in a given dataframe are used
to predict a third variable from the same dataframe. The prediction strength is measured by
expected conditional entropies.
}
\details{
The expected conditional entropy given by\cr

\emph{EH(Z|X,Y) = H(X,Y,Z) - H(X, Y)} \cr

measures the prediction uncertainty when pairs of variables \emph{X} and \emph{Y}
are used to predict variable \emph{Z}.
The lower the value of \emph{EH} given different pairs of variables \emph{X} and \emph{Y},
the stronger is the prediction of \emph{Z}.
}
\examples{
# use internal data set
data(lawdata)
df.att <- lawdata[[4]]

# three steps of data editing:
# 1. categorize variables 'years' and 'age' based on
# approximately three equally size groups (values based on cdf)
# 2. make sure all outcomes start from the value 0 (optional)
# 3. remove variable 'senior' as it consists of only unique values (thus redundant)
df.att.ed <- data.frame(
    status = df.att$status,
    gender = df.att$gender,
    office = df.att$office - 1,
    years = ifelse(df.att$years <= 3, 0,
        ifelse(df.att$years <= 13, 1, 2)
    ),
    age = ifelse(df.att$age <= 35, 0,
        ifelse(df.att$age <= 45, 1, 2)
    ),
    practice = df.att$practice,
    lawschool = df.att$lawschool - 1
)

# power of predicting 'status' using pairs of other variables
prediction_power("status", df.att.ed)
}
\references{
Frank, O., & Shafie, T. (2016). Multivariate entropy analysis of network data.
\emph{Bulletin of Sociological Methodology/Bulletin de Méthodologie Sociologique}, 129(1), 45-63.
}
\seealso{
\code{\link{entropy_trivar}}, \code{\link{entropy_bivar}}
}
\author{
Termeh Shafie
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/entropy_trivar.R
\name{entropy_trivar}
\alias{entropy_trivar}
\title{Trivariate Entropy}
\usage{
entropy_trivar(dat)
}
\arguments{
\item{dat}{dataframe with rows as observations and columns as variables.
Variables must all be observed or transformed categorical with finite range spaces.}
}
\value{
Dataframe with the first three columns representing possible triples
of variables (\code{X}, \code{Y}, \code{Z}) and the fourth column giving
trivariate entropies \code{H(X,Y,Z)}.
}
\description{
Computes trivariate entropies of all triples of discrete
variables in a multivariate data set.
}
\details{
Trivariate entropies can be used to check for functional relationships
and stochastic independence between triples of variables.

The trivariate entropy \emph{H(X,Y,Z)} of three discrete random variables
\emph{X}, \emph{Y}, and \emph{Z} is bounded according to
\deqn{H(X,Y) <= H(X,Y,Z) <= H(X,Z) + H(Y,Z) - H(Z).}

The increment between the trivariate entropy and its lower bound is equal to
the expected conditional entropy.
}
\examples{
# use internal data set
data(lawdata)
df_att <- lawdata[[4]]

# data editing:
# 1. categorize variables 'years' and 'age' into approximately
# equally sized groups
# 2. recode selected variables so categories start at 0
att_var <- data.frame(
  status    = df_att$status - 1,
  gender    = df_att$gender,
  office    = df_att$office - 1,
  years     = ifelse(df_att$years <= 3, 0,
                ifelse(df_att$years <= 13, 1, 2)),
  age       = ifelse(df_att$age <= 35, 0,
                ifelse(df_att$age <= 45, 1, 2)),
  practice  = df_att$practice,
  lawschool = df_att$lawschool - 1
)

# calculate trivariate entropies
h_trivar <- entropy_trivar(att_var)
}
\references{
Frank, O., & Shafie, T. (2016). Multivariate entropy analysis of network data.
\emph{Bulletin of Sociological Methodology/Bulletin de Méthodologie Sociologique}, 129(1), 45-63.
}
\seealso{
\code{\link{entropy_bivar}}, \code{\link{prediction_power}}
}
\author{
Termeh Shafie
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/assoc_graph.R
\name{assoc_graph}
\alias{assoc_graph}
\title{Association Graphs}
\usage{
assoc_graph(dat, cutoff = 0)
}
\arguments{
\item{dat}{dataframe with rows as observations and columns as variables. Variables must all be observed or transformed categorical with finite range spaces.}

\item{cutoff}{the cutoff point for the edges to be drawn based on joint entropies. Default is 0 and draws all edges.}
}
\value{
A ggraph object with nodes representing all variables in \code{dat} and edges
representing (the strength of) associations between them based on joint entropies.
}
\description{
Draws association graphs (graphical models) based on joint entropy values
to detect and visualize different dependence structures among the variables in the dataframe.
}
\details{
Draws association graphs based on given thresholds of joint entropy values
between pairs of variables represented as nodes. Thickness of edges between pairs of nodes/variables
indicates the strength of dependence between them. Isolated nodes are completely
independent and paths through certain nodes/variables indicate conditional dependencies.
}
\examples{
library(ggraph)
# use internal data set
data(lawdata)
df.att <- lawdata[[4]]

# three steps of data editing:
# 1. categorize variables 'years' and 'age' based on
# approximately three equally size groups (values based on cdf)
# 2. make sure all outcomes start from the value 0 (optional)
# 3. remove variable 'senior' as it consists of only unique values (thus redundant)
df.att.ed <- data.frame(
    status = df.att$status,
    gender = df.att$gender,
    office = df.att$office - 1,
    years = ifelse(df.att$years <= 3, 0,
        ifelse(df.att$years <= 13, 1, 2)
    ),
    age = ifelse(df.att$age <= 35, 0,
        ifelse(df.att$age <= 45, 1, 2)
    ),
    practice = df.att$practice,
    lawschool = df.att$lawschool - 1
)

# association graph based on cutoff 0.15
assoc_graph(df.att.ed, 0.15)
}
\references{
Frank, O., & Shafie, T. (2016). Multivariate entropy analysis of network data.
\emph{Bulletin of Sociological Methodology/Bulletin de Méthodologie Sociologique}, 129(1), 45-63.
}
\seealso{
\code{\link{joint_entropy}}
}
\author{
Termeh Shafie
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_triad_var.R
\name{get_triad_var}
\alias{get_triad_var}
\title{Get Triad Variables}
\usage{
get_triad_var(var, type = "att")
}
\arguments{
\item{var}{variable vector (actor attribute) or adjacency matrix (ties)
to be transformed to a triad variable.}

\item{type}{either 'att' for actor attribute (default) or 'tie' for relations.}
}
\value{
Dataframe with four columns:
first three columns show the vertex triad \code{u}, \code{v}, \code{w} ,
and the fourth column gives the value of the transformed triadic variable \code{var}.
}
\description{
Transforms vertex variables or observed directed/undirected ties
into triad variables.
}
\details{
For actor attributes, unique triples of original attribute values
constitute the outcome space. Note that the actor
attributes need to be categorical with finite range spaces.
For example, binary attributes have 8 possible triadic outcomes
(0,0,0),(1,0,0),(0,1,0),(1,1,0),(0,0,1),(1,0,1),(0,1,1),(1,1,1)
which are coded 0-7.
Warning message is shown if actor attribute has too many unique outcomes
as it will yield too many possible outcomes once converted in to a triad variable.
\cr

For directed relations, a sequence of indicators of length 6 created from the adjacency matrix
constitutes the 64 outcomes representing possible combinations of sending and receiving ties.
\cr

For undirected relations, triples of indicators are created from the adjacency matrix.
\cr
}
\examples{
# use internal data set
data(lawdata)
adj.advice <- lawdata[[1]]
adj.cowork <- lawdata[[3]]
df.att <- lawdata[[4]]

# three steps of data editing:
# 1. categorize variables 'years' and 'age' based on
# approximately three equally size groups (values based on cdf)
# 2. make sure all outcomes start from the value 0 (optional)
# 3. remove variable 'senior' as it consists of only unique values (thus redundant)
df.att.ed <- data.frame(
    status = df.att$status,
    gender = df.att$gender,
    office = df.att$office - 1,
    years = ifelse(df.att$years <= 3, 0,
        ifelse(df.att$years <= 13, 1, 2)
    ),
    age = ifelse(df.att$age <= 35, 0,
        ifelse(df.att$age <= 45, 1, 2)
    ),
    practice = df.att$practice,
    lawschool = df.att$lawschool - 1
)

# actor attribute converted to triad variable
triad.gend <- get_triad_var(df.att.ed$gender, "att")

# directed tie converted to triad variable
triad.adv <- get_triad_var(adj.advice, type = "tie")

# undirected tie converted to triad variable
triad.cwk <- get_triad_var(adj.cowork, type = "tie")
}
\references{
Frank, O., & Shafie, T. (2016). Multivariate entropy analysis of network data.
\emph{Bulletin of Sociological Methodology/Bulletin de Méthodologie Sociologique}, 129(1), 45-63.
}
\seealso{
\code{\link{get_dyad_var}}
}
\author{
Termeh Shafie
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/make_pred_plot.R
\name{make_pred_plot}
\alias{make_pred_plot}
\title{Prediction Power Heatmap}
\usage{
make_pred_plot(mat, title, low = "azure4", high = "white", text_size = 2.5)
}
\arguments{
\item{mat}{matrix returned by \code{\link{prediction_power}}. Entries should
contain expected conditional entropies \emph{EH(Z|X,Y)}.}

\item{title}{character string giving the plot title.}

\item{low}{color for low expected conditional entropy values. Default is
\code{"steelblue"}.}

\item{high}{color for high expected conditional entropy values. Default is
\code{"white"}.}

\item{text_size}{numeric value controlling the size of the cell labels.
Default is 2.5.}
}
\value{
A \code{ggplot} object showing a heatmap of expected conditional
entropy values. Darker cells indicate lower prediction uncertainty and
therefore higher prediction power.
}
\description{
Creates a heatmap for visualizing prediction power from a
prediction power matrix.
}
\details{
The plot visualizes expected conditional entropies
\deqn{EH(Z|X,Y)}
where \emph{Z} is the target variable and \emph{X} and \emph{Y} are predictors.
Diagonal entries correspond to prediction using a single predictor,
\emph{EH(Z|X)}, while off-diagonal entries correspond to prediction using
pairs of predictors, \emph{EH(Z|X,Y)}. Lower values indicate stronger
predictive power.
}
\examples{
# use internal data set
data(lawdata)

# extract node attributes
df_att <- lawdata[[4]]

# data editing:
# 1. discretize 'years' and 'age' into 3 categories
# 2. ensure values start at 0 where needed
att_var <- data.frame(
  status    = df_att$status - 1,
  gender    = df_att$gender,
  office    = df_att$office - 1,
  years     = ifelse(df_att$years <= 3, 0,
                ifelse(df_att$years <= 13, 1, 2)),
  age       = ifelse(df_att$age <= 35, 0,
                ifelse(df_att$age <= 45, 1, 2)),
  practice  = df_att$practice,
  lawschool = df_att$lawschool - 1
)

# compute prediction power matrix for 'status'
pred_mat <- prediction_power("status", att_var)

# visualize prediction power
make_pred_plot(pred_mat, "Prediction Power for Status")
}
\seealso{
\code{\link{prediction_power}}, \code{\link{entropy_trivar}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/entropy_bivar.R
\name{entropy_bivar}
\alias{entropy_bivar}
\title{Bivariate Entropy}
\usage{
entropy_bivar(dat)
}
\arguments{
\item{dat}{dataframe with rows as observations and columns as variables. Variables must all be observed or transformed categorical with finite range spaces.}
}
\value{
Upper triangular matrix giving bivariate entropies between pairs of variables given as
rows and columns of the matrix. The univariate entropies are given in the diagonal.
}
\description{
Computes the bivariate entropies between all pairs of (discrete) variables in a multivariate data set.
}
\details{
The bivariate entropy \emph{H(X,Y)} of two discrete random variables \emph{X} and \emph{Y}
can be used to check for functional relationships and stochastic independence between pairs of variables.
The bivariate entropy is bounded according to \cr

\emph{H(X) <= H(X,Y) <= H(X) + H(Y)}
\cr

where \emph{H(X)} and
\emph{H(Y)} are the univariate entropies.
}
\examples{
# use internal data set
data(lawdata)
df.att <- lawdata[[4]]

# three steps of data editing:
# 1. categorize variables 'years' and 'age' based on
# approximately three equally size groups (values based on cdf)
# 2. make sure all outcomes start from the value 0 (optional)
# 3. remove variable 'senior' as it consists of only unique values (thus redundant)
df.att.ed <- data.frame(
    status = df.att$status,
    gender = df.att$gender,
    office = df.att$office - 1,
    years = ifelse(df.att$years <= 3, 0,
        ifelse(df.att$years <= 13, 1, 2)
    ),
    age = ifelse(df.att$age <= 35, 0,
        ifelse(df.att$age <= 45, 1, 2)
    ),
    practice = df.att$practice,
    lawschool = df.att$lawschool - 1
)

# calculate bivariate entropies
H.biv <- entropy_bivar(df.att.ed)
# univariate entropies are then given as
diag(H.biv)
}
\references{
Frank, O., & Shafie, T. (2016). Multivariate entropy analysis of network data.
\emph{Bulletin of Sociological Methodology/Bulletin de Méthodologie Sociologique}, 129(1), 45-63.
}
\seealso{
\code{\link{joint_entropy}},  \code{\link{entropy_trivar}}, \code{\link{redundancy}}, \code{\link{prediction_power}}
}
\author{
Termeh Shafie
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/redundancy.R
\name{redundancy}
\alias{redundancy}
\title{Redundant Variables & Dimensionality Reduction}
\usage{
redundancy(dat, dec = 3)
}
\arguments{
\item{dat}{dataframe with rows as observations and columns as variables.
Variables must all be observed or transformed categorical with finite range spaces.}

\item{dec}{the precision given as number of decimals used to round bivariate entropies
in order to find redundant variables (the more decimals, the harder to detect redundancy). Default is 3.}
}
\value{
Binary matrix indicating which row and column variables hold the same information.
}
\description{
Finds redundant variables in a dataframe consisting of discrete variables.
}
\details{
Redundancy is defined as two variables holding the same information (bivariate entropies)
as at least one of the variable alone (univariate entropies).
Consider removing one of these two variable from the dataframe for further analysis.
}
\examples{
# use internal data set
data(lawdata)
df.att <- lawdata[[4]]

# two steps of data editing:
# 1. categorize variables 'years' and 'age' based on
# approximately three equally size groups (values based on cdf)
# 2. make sure all outcomes start from the value 0 (optional)
df.att.ed <- data.frame(
    senior = df.att$senior,
    status = df.att$status,
    gender = df.att$gender,
    office = df.att$office - 1,
    years = ifelse(df.att$years <= 3, 0,
        ifelse(df.att$years <= 13, 1, 2)
    ),
    age = ifelse(df.att$age <= 35, 0,
        ifelse(df.att$age <= 45, 1, 2)
    ),
    practice = df.att$practice,
    lawschool = df.att$lawschool - 1
)

# find redundant variables in dataframe
redundancy(df.att.ed) # variable 'senior' should be omitted
}
\references{
Frank, O., & Shafie, T. (2016). Multivariate entropy analysis of network data.
\emph{Bulletin of Sociological Methodology/Bulletin de Méthodologie Sociologique}, 129(1), 45-63.
}
\seealso{
\code{\link{entropy_bivar}},
}
\author{
Termeh Shafie
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_dyad_var.R
\name{get_dyad_var}
\alias{get_dyad_var}
\title{Get Dyad Variables}
\usage{
get_dyad_var(var, type = "att")
}
\arguments{
\item{var}{variable vector (actor attribute) or adjacency matrix (ties)
to be transformed to a dyad variable.}

\item{type}{either 'att' for actor attribute (default) or 'tie' for relations.}
}
\value{
Dataframe with three columns:
first two columns show the vertex pairs \code{u} and \code{v} where \code{u<v} ,
and the third column gives the value of the transformed dyadic variable \code{var}.
}
\description{
Transforms vertex variables or observed directed/undirected ties
into dyad variables.
}
\details{
Dyad variables are given as pairs of incident vertex variables
or actor attributes. Here, unique pairs of original attribute values
constitute the outcome space. Note that the actor  attributes need
to be categorical with finite range spaces. For example, binary
attribute yields outcome space (0,0), (0,1), (1,0), (1,1) coded as (0),(1),(2),(3).
Warning message is shown if actor attribute has too many unique outcomes
as it will yield too many possible outcomes once converted in to a dyad variable.
\cr

For directed relations, pairs of indicators from the adjacency matrix constitute
the four outcomes representing possible combinations of sending and receiving ties:
(0,0), (0,1), (1,0), (1,1) coded as (0),(1),(2),(3).
\cr

For undirected relations, an indicator variable which is directly read from the
adjacency matrix represents the dyadic variable.
\cr
}
\examples{
# use internal data set
data(lawdata)
adj.advice <- lawdata[[1]]
adj.cowork <- lawdata[[3]]
df.att <- lawdata[[4]]

# three steps of data editing of attribute dataframe:
# 1. categorize variables 'years' and 'age' based on
# approximately three equally size groups (values based on cdf)
# 2. make sure all outcomes start from the value 0 (optional)
# 3. remove variable 'senior' as it consists of only unique values (thus redundant)
df.att.ed <- data.frame(
    status = df.att$status,
    gender = df.att$gender,
    office = df.att$office - 1,
    years = ifelse(df.att$years <= 3, 0,
        ifelse(df.att$years <= 13, 1, 2)
    ),
    age = ifelse(df.att$age <= 35, 0,
        ifelse(df.att$age <= 45, 1, 2)
    ),
    practice = df.att$practice,
    lawschool = df.att$lawschool - 1
)

# actor attribute converted to dyad variable
dyad.gend <- get_dyad_var(df.att.ed$gender, "att")

# directed tie converted to dyad variable
dyad.adv <- get_dyad_var(adj.advice, "tie")

# undirected tie converted to dyad variable
dyad.cwk <- get_dyad_var(adj.cowork, "tie")
}
\references{
Frank, O., & Shafie, T. (2016). Multivariate entropy analysis of network data.
\emph{Bulletin of Sociological Methodology/Bulletin de Méthodologie Sociologique}, 129(1), 45-63.
}
\seealso{
\code{\link{get_triad_var}}
}
\author{
Termeh Shafie
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/entropy_tetravar.R
\name{entropy_tetravar}
\alias{entropy_tetravar}
\title{Tetravariate Entropy}
\usage{
entropy_tetravar(dat, dec = 2)
}
\arguments{
\item{dat}{dataframe with rows as observations and columns as variables.
Variables must be categorical with finite range spaces.}

\item{dec}{number of decimals used for rounding the entropy values.
Default is 2.}
}
\value{
A dataframe with one row for each ordered decomposition of four
variables into predictors and conditioning variables. The columns are:
\item{X}{first variable in the pair of interest.}
\item{Y}{second variable in the pair of interest.}
\item{Z}{first conditioning variable.}
\item{U}{second conditioning variable.}
\item{H_XYZU}{tetravariate entropy \emph{H(X,Y,Z,U)}.}
\item{EH_U_XYZ}{expected conditional entropy \emph{EH(U|X,Y,Z)}.}
\item{EH_Z_XYU}{expected conditional entropy \emph{EH(Z|X,Y,U)}.}
\item{EJ_XY_ZU}{expected conditional joint entropy \emph{EJ(X,Y|Z,U)}.}
}
\description{
Computes tetravariate entropies, expected conditional entropies,
and expected conditional joint entropies for all quadruples of variables in a
multivariate discrete data set.
}
\details{
For four variables \emph{X}, \emph{Y}, \emph{Z}, and \emph{U}, the
tetravariate entropy is denoted \emph{H(X,Y,Z,U)}. The expected conditional
entropies are computed as
\deqn{EH(U|X,Y,Z) = H(X,Y,Z,U) - H(X,Y,Z)}
and
\deqn{EH(Z|X,Y,U) = H(X,Y,Z,U) - H(X,Y,U).}
The expected conditional joint entropy is computed as
\deqn{EJ(X,Y|Z,U) = H(X,Z,U) + H(Y,Z,U) - H(Z,U) - H(X,Y,Z,U).}
This quantity measures deviation from conditional independence of the form
\eqn{X \perp Y \,\vert\, Z, U}{X is independent of Y given Z, U}.
Smaller values indicate weaker conditional dependence.
}
\examples{
# use internal data set
data(lawdata)

# extract node attributes
df_att <- lawdata[[4]]

# data editing:
# 1. discretize 'years' and 'age' into three approximately balanced groups
# 2. recode selected variables so categories start at 0
att_var <- data.frame(
  status    = df_att$status - 1,
  gender    = df_att$gender,
  office    = df_att$office - 1,
  years     = ifelse(df_att$years <= 3, 0,
                ifelse(df_att$years <= 13, 1, 2)),
  age       = ifelse(df_att$age <= 35, 0,
                ifelse(df_att$age <= 45, 1, 2)),
  practice  = df_att$practice,
  lawschool = df_att$lawschool - 1
)

# compute tetravariate entropy quantities for five selected variables
entropy_tetravar(
  dat = att_var[, c("gender", "years", "age", "office", "practice")]
)

}
\references{
Frank, O., & Shafie, T. (2016). Multivariate entropy analysis of
network data. \emph{Bulletin of Sociological Methodology/Bulletin de
Méthodologie Sociologique}, 129(1), 45-63.
}
\seealso{
\code{\link{entropy_trivar}}, \code{\link{entropy_bivar}},
\code{\link{prediction_power}}
}
\author{
Termeh Shafie
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/joint_entropy.R
\name{joint_entropy}
\alias{joint_entropy}
\title{Joint Entropy}
\usage{
joint_entropy(dat, dec = 3)
}
\arguments{
\item{dat}{dataframe with rows as observations and columns as variables.
Variables must all be observed or transformed categorical with finite range spaces.}

\item{dec}{the precision given in number of decimals for which
the frequency distribution of unique entropy values is created. Default is 3.}
}
\value{
List with
\item{matrix}{an upper triangular joint entropy matrix (univariate entropies in the diagonal).}
\item{freq}{a dataframe giving the frequency distributions of unique joint entropy values.}
}
\description{
Computes the joint entropies between all pairs of (discrete)
variables in a multivariate data set.
}
\details{
The joint entropy \emph{J(X,Y)} of discrete variables \emph{X} and \emph{Y}
is a measure of dependence or association between them, defined as
\cr

\emph{J(X,Y) = H(X) + H(Y) - H(X,Y)}.
\cr

Two variables are independent if their joint entropy,
i.e. their mutual information, is equal to zero.
The frequency distributions can be used to decide upon convenient thresholds for
constructing association graphs.
}
\examples{
# use internal data set
data(lawdata)
df.att <- lawdata[[4]]

# three steps of data editing:
# 1. categorize variables 'years' and 'age' based on
# approximately three equally size groups (values based on cdf)
# 2. make sure all outcomes start from the value 0 (optional)
# 3. remove variable 'senior' as it consists of only unique values (thus redundant)
df.att.ed <- data.frame(
    status = df.att$status,
    gender = df.att$gender,
    office = df.att$office - 1,
    years = ifelse(df.att$years <= 3, 0,
        ifelse(df.att$years <= 13, 1, 2)
    ),
    age = ifelse(df.att$age <= 35, 0,
        ifelse(df.att$age <= 45, 1, 2)
    ),
    practice = df.att$practice,
    lawschool = df.att$lawschool - 1
)

# calculate joint entropies
J <- joint_entropy(df.att.ed)
# joint entropy matrix
J$matrix
# frequency distribution of joint entropy values
J$freq
}
\references{
Frank, O., & Shafie, T. (2016). Multivariate entropy analysis of network data.
\emph{Bulletin of Sociological Methodology/Bulletin de Méthodologie Sociologique}, 129(1), 45-63.
}
\seealso{
\code{\link{assoc_graph}}, \code{\link{entropy_bivar}}
}
\author{
Termeh Shafie
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/div_gof.R
\name{div_gof}
\alias{div_gof}
\title{Divergence Tests of Goodness of Fit}
\usage{
div_gof(
  dat,
  var_uniform = NULL,
  var1 = NULL,
  var2 = NULL,
  var_cond = NULL,
  model_full = NULL,
  model_reduced = NULL,
  alpha = 0.05,
  dec = 3,
  use_approx_cv = TRUE
)
}
\arguments{
\item{dat}{dataframe with rows as observations and columns as variables.
Variables must be categorical with finite range spaces.}

\item{var_uniform}{character name of a variable in \code{dat} to test for uniformity.}

\item{var1}{character name of the first variable.}

\item{var2}{character name of the second variable.}

\item{var_cond}{optional character vector of conditioning variables.}

\item{model_full}{list containing \code{D} and \code{df} for the full model.}

\item{model_reduced}{list containing \code{D} and \code{df} for the reduced model.}

\item{alpha}{significance level. Default is 0.05.}

\item{dec}{number of decimals for rounding. Default is 3.}

\item{use_approx_cv}{logical; if \code{TRUE}, uses the approximate critical value
\code{df + sqrt(8 * df)}. If \code{FALSE}, uses the chi-square quantile.}
}
\value{
Dataframe with test type, divergence \emph{D}, chi-square statistic,
degrees of freedom, critical value, and decision.
}
\description{
Performs divergence-based goodness-of-fit tests for discrete data,
including tests of uniformity, pairwise independence, conditional independence,
and nested model comparisons.
}
\details{
The function implements four types of tests:

\strong{1. Uniformity}
\deqn{D = \log r_X - H(X)}

\strong{2. Pairwise Independence}
\deqn{D = H(X) + H(Y) - H(X,Y)}

\strong{3. Conditional Independence}
\deqn{D = H(X,Z) + H(Y,Z) - H(Z) - H(X,Y,Z)}

where \emph{Z} may also represent a vector of conditioning variables.

\strong{4. Nested Model Comparison}
\deqn{D = D_{reduced} - D_{full}}

The test statistic is
\deqn{2nD\log(2),}
since entropies are computed using base 2 logarithms.

Smaller divergence values indicate better model fit.
}
\examples{
data(lawdata)
df_att <- lawdata[[4]]

att_var <- data.frame(
  status    = df_att$status - 1,
  gender    = df_att$gender,
  office    = df_att$office - 1,
  years     = ifelse(df_att$years <= 3, 0,
                ifelse(df_att$years <= 13, 1, 2)),
  age       = ifelse(df_att$age <= 35, 0,
                ifelse(df_att$age <= 45, 1, 2)),
  practice  = df_att$practice,
  lawschool = df_att$lawschool - 1
)

## 1. Test uniformity
div_gof(att_var, var_uniform = "gender")

## 2. Test pairwise independence
div_gof(att_var, var1 = "status", var2 = "gender")

## 3. Test conditional independence

## (a) Conditional independence given a single variable
div_gof(att_var,
        var1 = "status",
        var2 = "gender",
        var_cond = "years")

## (b) Conditional independence given multiple variables
div_gof(att_var,
        var1 = "status",
        var2 = "gender",
        var_cond = c("years", "age"))

## 4. Nested model comparison
## Compare reduced models against the saturated empirical model.
## The saturated model has divergence D = 0 and df = 0.
m_full <- list(D = 0, df = 0)

## (a) Pairwise independence model
m_reduced <- div_gof(att_var,
                    var1 = "status",
                    var2 = "gender")

div_gof(att_var,
        model_full = m_full,
        model_reduced = list(D = m_reduced$D, df = m_reduced$df))

## (b) Conditional independence model
m_reduced <- div_gof(att_var,
                    var1 = "status",
                    var2 = "gender",
                    var_cond = "years")

div_gof(att_var,
        model_full = m_full,
        model_reduced = list(D = m_reduced$D, df = m_reduced$df))

## 5. Nested comparison against the saturated empirical model
m_full <- list(D = 0, df = 0)

m_reduced <- div_gof(att_var,
                    var1 = "status",
                    var2 = "gender")

div_gof(att_var,
        model_full = m_full,
        model_reduced = list(D = m_reduced$D, df = m_reduced$df))
}
\references{
Frank, O., & Shafie, T. (2016). Multivariate entropy analysis of network data.
\emph{Bulletin of Sociological Methodology/Bulletin de Méthodologie Sociologique}, 129(1), 45-63.
}
\seealso{
\code{\link{joint_entropy}}, \code{\link{entropy_trivar}}
}
\author{
Termeh Shafie
}
